# sage_setup: distribution = sagemath-modules
# distutils: libraries = GSL_LIBRARIES
# distutils: library_dirs = GSL_LIBDIR
# distutils: include_dirs = GSL_INCDIR
from sage.libs.gsl.types cimport *

cdef extern from "gsl/gsl_poly.h":

  #  Evaluate polynomial
  double  gsl_poly_eval(double c[], int len, double x)

  # divided-difference polynomials
  int  gsl_poly_dd_init(double dd[], double xa[], double ya[], size_t size)

  double  gsl_poly_dd_eval(double dd[], double xa[], size_t size, double x)

  int  gsl_poly_dd_taylor(double c[], double xp, double dd[], double xa[], size_t size, double w[])

  # quadratic equation
  int  gsl_poly_solve_quadratic(double a, double b, double c, double *x0, double *x1)

  int  gsl_poly_complex_solve_quadratic(double a, double b, double c, gsl_complex *z0, gsl_complex *z1)

  # cubic equation
  int  gsl_poly_solve_cubic(double a, double b, double c, double *x0, double *x1, double *x2)

  int  gsl_poly_complex_solve_cubic(double a, double b, double c, gsl_complex *z0, gsl_complex *z1, gsl_complex *z2)

  # General Polynomial Equations
  ctypedef struct gsl_poly_complex_workspace:
    size_t nc
    double * matrix

  gsl_poly_complex_workspace *  gsl_poly_complex_workspace_alloc(size_t n)

  void  gsl_poly_complex_workspace_free(gsl_poly_complex_workspace * w)

  int  gsl_poly_complex_solve(double * a, size_t n, gsl_poly_complex_workspace * w, gsl_complex_packed_ptr z)





# sage_setup: distribution = sagemath-modules
r"""
Special methods for matrices over discrete valuation rings/fields.
"""

# ****************************************************************************
#       Copyright (C) 2020 Xavier Caruso <xavier.caruso@normalesup.org>
#                          Raphaël Pagès <raphael.pages@u-bordeaux.fr>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#          https://www.gnu.org/licenses/
# ****************************************************************************


from sage.matrix.matrix_generic_dense cimport Matrix_generic_dense
from sage.structure.element cimport RingElement
from sage.rings.infinity import Infinity


# We assume that H is square
cpdef hessenbergize_cdvf(Matrix_generic_dense H):
    r"""
    Replace `H` with a Hessenberg form of it.

    .. NOTE::

        This function assumes that H is a matrix over
        a complete discrete valuation field.

        The pivot on each column is always chosen
        with maximal relative precision, which ensures
        the numerical stability of the algorithm.

    TESTS::

        sage: # needs sage.rings.padics
        sage: K = Qp(5, print_mode='digits', prec=5)
        sage: H = matrix(K, 3, 3, range(9))
        sage: H
        [        0  ...00001  ...00002]
        [ ...00003  ...00004 ...000010]
        [ ...00011  ...00012  ...00013]
        sage: H.hessenbergize()
        sage: H
        [        0  ...00010  ...00002]
        [ ...00003  ...00024 ...000010]
        [ ...00000  ...44440  ...44443]

    ::

        sage: M = random_matrix(K, 6, 6)                                                # needs sage.rings.padics
        sage: M.charpoly()[0] == M.determinant()                                        # needs sage.rings.padics
        True

    We check that :issue:`31753` is resolved::

        sage: R.<t> = GF(5)[[]]
        sage: M = matrix(3, 3, [ 1, t + O(t^3), t^2,
        ....:                    1 + t + O(t^3), 2 + t^2, 3 + 2*t + O(t^3),
        ....:                    t - t^2, 2*t, 1 + t ])
        sage: M.charpoly()
        x^3 + (1 + 4*t + 4*t^2 + O(t^3))*x^2 + (t + 2*t^2 + O(t^3))*x + 3 + 2*t^2 + O(t^3)
    """
    cdef Py_ssize_t n, i, j, k
    cdef RingElement entry, pivot, inv, scalar

    n = H.nrows()
    for j in range(n-1):
        k = j + 1
        maxi = H.get_unsafe(k, j).precision_relative()
        i = j + 2
        while maxi is not Infinity and i < n:
            entry = H.get_unsafe(i, j)
            if entry:
                m = entry.precision_relative()
                if m > maxi:
                    maxi = m
                    k = i
            i += 1

        if k != j + 1:
            H.swap_rows_c(j+1, k)
            H.swap_columns_c(j+1, k)
        pivot = H.get_unsafe(j+1, j)
        if pivot:
            inv = ~pivot
            for i in range(j+2, n):
                scalar = inv * H.get_unsafe(i, j)
                H.add_multiple_of_row_c(i, j+1, -scalar, j)
                H.add_multiple_of_column_c(j+1, i, scalar, 0)



# sage_setup: distribution = sagemath-pari
"""
This file provides the declaration for ``CRElement`` and the morphisms
to and from the integers and rationals.

It is included in the .pxd files associated to gluing files, such as
padic_capped_relative_element.pxd.

AUTHORS:

- David Roe (2012-03-01) -- initial version
"""

#*****************************************************************************
#       Copyright (C) 2012 David Roe <roed.math@gmail.com>
#                          William Stein <wstein@gmail.com>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#  as published by the Free Software Foundation; either version 2 of
#  the License, or (at your option) any later version.
#
#                  http://www.gnu.org/licenses/
#*****************************************************************************

# includes the header for pAdicTemplateElement
include "padic_template_element_header.pxi"

from sage.categories.morphism cimport Morphism
from sage.rings.morphism cimport RingHomomorphism, RingMap

cdef class CRElement(pAdicTemplateElement):
    cdef celement unit
    cdef long ordp
    cdef long relprec

    cdef CRElement _new_c(self)
    cdef int _normalize(self) except -1
    cpdef val_unit(self, p=*)

cdef class pAdicCoercion_ZZ_CR(RingHomomorphism):
    cdef CRElement _zero
    cdef RingMap _section
cdef class pAdicConvert_CR_ZZ(RingMap):
    pass
cdef class pAdicCoercion_QQ_CR(RingHomomorphism):
    cdef CRElement _zero
    cdef RingMap _section
cdef class pAdicConvert_CR_QQ(RingMap):
    pass
cdef class pAdicConvert_QQ_CR(Morphism):
    cdef CRElement _zero
    cdef RingMap _section
cdef class pAdicCoercion_CR_frac_field(RingHomomorphism):
    cdef CRElement _zero
    cdef Morphism _section
cdef class pAdicConvert_CR_frac_field(Morphism):
    cdef CRElement _zero



# sage_setup: distribution = sagemath-schemes
from sage.schemes.hyperelliptic_curves.constructor import HyperellipticCurve
from sage.schemes.hyperelliptic_curves.kummer_surface import KummerSurface
from sage.schemes.hyperelliptic_curves.mestre import (
    Mestre_conic, HyperellipticCurve_from_invariants)
from sage.schemes.hyperelliptic_curves import monsky_washnitzer



# sage_setup: distribution = sagemath-schemes
from sage.misc.lazy_import import lazy_import

from sage.dynamics.arithmetic_dynamics.generic_ds import DynamicalSystem
from sage.dynamics.arithmetic_dynamics.affine_ds import DynamicalSystem_affine
from sage.dynamics.arithmetic_dynamics.projective_ds import DynamicalSystem_projective
from sage.dynamics.arithmetic_dynamics.product_projective_ds import DynamicalSystem_product_projective
from sage.dynamics.arithmetic_dynamics.berkovich_ds import DynamicalSystem_Berkovich
from sage.dynamics.arithmetic_dynamics.dynamical_semigroup import DynamicalSemigroup
from sage.dynamics.arithmetic_dynamics.dynamical_semigroup import DynamicalSemigroup_affine
from sage.dynamics.arithmetic_dynamics.dynamical_semigroup import DynamicalSemigroup_projective
lazy_import('sage.dynamics.arithmetic_dynamics.wehlerK3', 'WehlerK3Surface')
lazy_import('sage.dynamics.arithmetic_dynamics.wehlerK3', 'random_WehlerK3Surface')
del lazy_import



# sage_setup: distribution = sagemath-objects
from typing import Any

class ElementWrapper:
    def __init__(self, parent: Any, value: Any) -> None: ...
    def __getstate__(self) -> tuple[Any, dict[str, Any]]: ...
    def __setstate__(self, state: tuple[Any, dict[str, Any]]) -> None: ...
    def _repr_(self) -> str: ...
    def _latex_(self) -> str: ...
    def _ascii_art_(self) -> str: ...
    def _unicode_art_(self) -> str: ...
    def __hash__(self) -> int: ...
    def __richcmp__(self, right: Any, op: int) -> bool: ...
    def _richcmp_(self, right: Any, op: int) -> bool: ...
    def _lt_by_value(self, other: Any) -> bool: ...
    def __copy__(self) -> Any: ...

class DummyParent:
    def __init__(self, name: str) -> None: ...
    def _repr_(self) -> str: ...

class ElementWrapperTester(ElementWrapper):
    def __init__(self) -> None: ...
    def append(self, x: Any) -> None: ...
    def _repr_(self) -> str: ...

class ElementWrapperCheckWrappedClass(ElementWrapper):
    wrapped_class: Any
    def __hash__(self) -> int: ...
    def __richcmp__(self, right: Any, op: int) -> bool: ...



# sage_setup: distribution = sagemath-modules
r"""
Toric lattices

This module was designed as a part of the framework for toric varieties
(:mod:`~sage.schemes.toric.variety`,
:mod:`~sage.schemes.toric.fano_variety`).

All toric lattices are isomorphic to `\ZZ^n` for some `n`, but will prevent
you from doing "wrong" operations with objects from different lattices.

AUTHORS:

- Andrey Novoseltsev (2010-05-27): initial version.
- Andrey Novoseltsev (2010-07-30): sublattices and quotients.

EXAMPLES:

The simplest way to create a toric lattice is to specify its dimension only::

    sage: N = ToricLattice(3)
    sage: N
    3-d lattice N

While our lattice ``N`` is called exactly "N" it is a coincidence: all
lattices are called "N" by default::

    sage: another_name = ToricLattice(3)
    sage: another_name
    3-d lattice N

If fact, the above lattice is exactly the same as before as an object in
memory::

    sage: N is another_name
    True

There are actually four names associated to a toric lattice and they all must
be the same for two lattices to coincide::

    sage: N, N.dual(), latex(N), latex(N.dual())
    (3-d lattice N, 3-d lattice M, N, M)

Notice that the lattice dual to ``N`` is called "M" which is standard in toric
geometry. This happens only if you allow completely automatic handling of
names::

    sage: another_N = ToricLattice(3, "N")
    sage: another_N.dual()
    3-d lattice N*
    sage: N is another_N
    False

What can you do with toric lattices? Well, their main purpose is to allow
creation of elements of toric lattices::

    sage: n = N([1,2,3])
    sage: n
    N(1, 2, 3)
    sage: M = N.dual()
    sage: m = M(1,2,3)
    sage: m
    M(1, 2, 3)

Dual lattices can act on each other::

    sage: n * m
    14
    sage: m * n
    14

You can also add elements of the same lattice or scale them::

    sage: 2 * n
    N(2, 4, 6)
    sage: n * 2
    N(2, 4, 6)
    sage: n + n
    N(2, 4, 6)

However, you cannot "mix wrong lattices" in your expressions::

    sage: n + m
    Traceback (most recent call last):
    ...
    TypeError: unsupported operand parent(s) for +:
    '3-d lattice N' and '3-d lattice M'
    sage: n * n
    Traceback (most recent call last):
    ...
    TypeError: elements of the same toric lattice cannot be multiplied!
    sage: n == m
    False

Note that ``n`` and ``m`` are not equal to each other even though they are
both "just (1,2,3)." Moreover, you cannot easily convert elements between
toric lattices::

    sage: M(n)
    Traceback (most recent call last):
    ...
    TypeError: N(1, 2, 3) cannot be converted to 3-d lattice M!

If you really need to consider elements of one lattice as elements of another,
you can either use intermediate conversion to "just a vector"::

    sage: ZZ3 = ZZ^3
    sage: n_in_M = M(ZZ3(n))
    sage: n_in_M
    M(1, 2, 3)
    sage: n == n_in_M
    False
    sage: n_in_M == m
    True

Or you can create a homomorphism from one lattice to any other::

    sage: h = N.hom(identity_matrix(3), M)
    sage: h(n)
    M(1, 2, 3)

.. WARNING::

    While integer vectors (elements of `\ZZ^n`) are printed as ``(1,2,3)``,
    in the code ``(1,2,3)`` is a :class:`tuple`, which has nothing to do
    neither with vectors, nor with toric lattices, so the following is
    probably not what you want while working with toric geometry objects::

        sage: (1,2,3) + (1,2,3)
        (1, 2, 3, 1, 2, 3)

    Instead, use syntax like ::

        sage: N(1,2,3) + N(1,2,3)
        N(2, 4, 6)
"""
# Parts of the "tutorial" above are also in toric_lattice_element.pyx.


# ****************************************************************************
#       Copyright (C) 2010 Andrey Novoseltsev <novoselt@gmail.com>
#       Copyright (C) 2010 William Stein <wstein@gmail.com>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#
#                  https://www.gnu.org/licenses/
# ****************************************************************************

from sage.geometry.toric_lattice_element import ToricLatticeElement
from sage.misc.lazy_import import lazy_import
lazy_import('sage.geometry.toric_plotter', 'ToricPlotter')
from sage.misc.latex import latex
from sage.structure.element import parent
from sage.structure.richcmp import (richcmp_method, richcmp, rich_to_bool,
                                    richcmp_not_equal)
from sage.modules.fg_pid.fgp_element import FGP_Element
from sage.modules.fg_pid.fgp_module import FGP_Module_class
from sage.modules.free_module import (FreeModule_ambient_pid,
                                      FreeModule_generic_pid,
                                      FreeModule_submodule_pid,
                                      FreeModule_submodule_with_basis_pid)
from sage.rings.integer_ring import ZZ
from sage.rings.rational_field import QQ
from sage.structure.factory import UniqueFactory


class ToricLatticeFactory(UniqueFactory):
    r"""
    Create a lattice for toric geometry objects.

    INPUT:

    - ``rank`` -- nonnegative integer; the only mandatory parameter

    - ``name`` -- string

    - ``dual_name`` -- string

    - ``latex_name`` -- string

    - ``latex_dual_name`` -- string

    OUTPUT: lattice

    A toric lattice is uniquely determined by its rank and associated names.
    There are four such "associated names" whose meaning should be clear from
    the names of the corresponding parameters, but the choice of default
    values is a little bit involved. So here is the full description of the
    "naming algorithm":

    #. If no names were given at all, then this lattice will be called "N" and
       the dual one "M". These are the standard choices in toric geometry.

    #. If ``name`` was given and ``dual_name`` was not, then ``dual_name``
       will be ``name`` followed by "*".

    #. If LaTeX names were not given, they will coincide with the "usual"
       names, but if ``dual_name`` was constructed automatically, the trailing
       star will be typeset as a superscript.

    EXAMPLES:

    Let's start with no names at all and see how automatic names are given::

        sage: L1 = ToricLattice(3)
        sage: L1
        3-d lattice N
        sage: L1.dual()
        3-d lattice M

    If we give the name "N" explicitly, the dual lattice will be called "N*"::

        sage: L2 = ToricLattice(3, "N")
        sage: L2
        3-d lattice N
        sage: L2.dual()
        3-d lattice N*

    However, we can give an explicit name for it too::

        sage: L3 = ToricLattice(3, "N", "M")
        sage: L3
        3-d lattice N
        sage: L3.dual()
        3-d lattice M

    If you want, you may also give explicit LaTeX names::

        sage: L4 = ToricLattice(3, "N", "M", r"\mathbb{N}", r"\mathbb{M}")
        sage: latex(L4)
        \mathbb{N}
        sage: latex(L4.dual())
        \mathbb{M}

    While all four lattices above are called "N", only two of them are equal
    (and are actually the same)::

        sage: L1 == L2
        False
        sage: L1 == L3
        True
        sage: L1 is L3
        True
        sage: L1 == L4
        False

    The reason for this is that ``L2`` and ``L4`` have different names either
    for dual lattices or for LaTeX typesetting.
    """

    def create_key(self, rank, name=None, dual_name=None,
                   latex_name=None, latex_dual_name=None):
        """
        Create a key that uniquely identifies this toric lattice.

        See :class:`ToricLattice <ToricLatticeFactory>` for documentation.

        .. WARNING::

            You probably should not use this function directly.

        TESTS::

            sage: ToricLattice.create_key(3)
            (3, 'N', 'M', 'N', 'M')
            sage: N = ToricLattice(3)
            sage: loads(dumps(N)) is N
            True
            sage: TestSuite(N).run()
        """
        rank = int(rank)
        # Should we use standard M and N lattices?
        if name is None:
            if dual_name is not None:
                raise ValueError("you can name the dual lattice only if you "
                                 "also name the original one!")
            name = "N"
            dual_name = "M"
        if latex_name is None:
            latex_name = name
        # Now name and latex_name are set
        # The default for latex_dual_name depends on whether dual_name was
        # given or constructed, so we determine it before dual_name
        if latex_dual_name is None:
            latex_dual_name = (dual_name if dual_name is not None
                                         else latex_name + "^*")
        if dual_name is None:
            dual_name = name + "*"
        return (rank, name, dual_name, latex_name, latex_dual_name)

    def create_object(self, version, key):
        r"""
        Create the toric lattice described by ``key``.

        See :class:`ToricLattice <ToricLatticeFactory>` for documentation.

        .. WARNING::

            You probably should not use this function directly.

        TESTS::

            sage: key = ToricLattice.create_key(3)
            sage: ToricLattice.create_object(1, key)
            3-d lattice N
        """
        return ToricLattice_ambient(*key)


ToricLattice = ToricLatticeFactory("ToricLattice")


# Possible TODO's:
# - implement a better construction() method, which still will prohibit
#   operations mixing lattices by conversion to ZZ^n
# - maybe __call__ is not the right place to prohibit conversion between
#   lattices (we need it now so that morphisms behave nicely)
class ToricLattice_generic(FreeModule_generic_pid):
    r"""
    Abstract base class for toric lattices.
    """

    Element = ToricLatticeElement

    # It is not recommended to override __call__ in Parent-derived objects
    # since it may interfere with the coercion model. We do it here to allow
    # N(1,2,3) to be interpreted as N([1,2,3]). We also prohibit N(m) where
    # m is an element of another lattice. Otherwise morphisms will care only
    # about dimension of lattices.
    def __call__(self, *args, **kwds):
        r"""
        Construct a new element of ``self``.

        INPUT:

        - anything that can be interpreted as coordinates, except for elements
          of other lattices.

        OUTPUT: :class:`~sage.geometry.toric_lattice_element.ToricLatticeElement`

        TESTS::

            sage: N = ToricLattice(3)
            sage: N.__call__([1,2,3])
            N(1, 2, 3)
            sage: N([1,2,3])    # indirect test
            N(1, 2, 3)

        The point of overriding this function was to allow writing the above
        command as::

            sage: N(1,2,3)
            N(1, 2, 3)

        We also test that the special treatment of zero still works::

            sage: N(0)
            N(0, 0, 0)

        Quotients of toric lattices can be converted to a new toric
        lattice of the appropriate dimension::

            sage: N3 = ToricLattice(3, 'N3')
            sage: Q = N3 / N3.span([ N3(1,2,3) ])
            sage: Q.an_element()                                                        # needs sage.libs.pari
            N3[1, 0, 0]
            sage: N2 = ToricLattice(2, 'N2')
            sage: N2( Q.an_element() )                                                  # needs sage.libs.pari
            N2(1, 0)
        """
        supercall = super().__call__
        if args == (0, ):
            # Special treatment for N(0) to return (0,...,0)
            return supercall(*args, **kwds)

        if (isinstance(args[0], ToricLattice_quotient_element)
            and args[0].parent().is_torsion_free()):
            # convert a torsion free quotient lattice
            return supercall(list(args[0]), **kwds)

        try:
            coordinates = [ZZ(_) for _ in args]
        except TypeError:
            # Prohibit conversion of elements of other lattices
            if (isinstance(args[0], ToricLatticeElement)
                and args[0].parent().ambient_module()
                is not self.ambient_module()):
                raise TypeError("%s cannot be converted to %s!"
                                % (args[0], self))
            # "Standard call"
            return supercall(*args, **kwds)
        # Coordinates were given without packing them into a list or a tuple
        return supercall(coordinates, **kwds)

    def _coerce_map_from_(self, other):
        """
        Return a coercion map from ``other`` to ``self``, or None.

        This prevents the construction of coercion maps between
        lattices with different ambient modules, so :meth:`__call__`
        is invoked instead, which prohibits conversion::

            sage: N = ToricLattice(3)
            sage: M = N.dual()
            sage: M(N(1,2,3))
            Traceback (most recent call last):
            ...
            TypeError: N(1, 2, 3) cannot be converted to 3-d lattice M!
        """
        if (isinstance(other, ToricLattice_generic) and
            other.ambient_module() is not self.ambient_module()):
            return None
        return super()._convert_map_from_(other)

    def __contains__(self, point) -> bool:
        r"""
        Check if ``point`` is an element of ``self``.

        INPUT:

        - ``point`` -- anything

        OUTPUT: ``True`` if ``point`` is an element of ``self``, ``False``
        otherwise

        TESTS::

            sage: N = ToricLattice(3)
            sage: M = N.dual()
            sage: L = ToricLattice(3, "L")
            sage: 1 in N
            False
            sage: (1,0) in N
            False
            sage: (1,0,0) in N
            True
            sage: N(1,0,0) in N
            True
            sage: M(1,0,0) in N
            False
            sage: L(1,0,0) in N
            False
            sage: (1/2,0,0) in N
            False
            sage: (2/2,0,0) in N
            True
        """
        try:
            self(point)
        except TypeError:
            return False
        return True

    # We need to override this function, otherwise e.g. the sum of elements of
    # different lattices of the same dimension will live in ZZ^n.
    def construction(self):
        r"""
        Return the functorial construction of ``self``.

        OUTPUT:

        ``None``, we do not think of toric lattices as constructed from
        simpler objects since we do not want to perform arithmetic involving
        different lattices.

        TESTS::

            sage: print(ToricLattice(3).construction())
            None
        """
        return None

    def direct_sum(self, other):
        r"""
        Return the direct sum with ``other``.

        INPUT:

        - ``other`` -- a toric lattice or more general module

        OUTPUT:

        The direct sum of ``self`` and ``other`` as `\ZZ`-modules. If
        ``other`` is a :class:`ToricLattice <ToricLatticeFactory>`,
        another toric lattice will be returned.

        EXAMPLES::

            sage: K = ToricLattice(3, 'K')
            sage: L = ToricLattice(3, 'L')
            sage: N = K.direct_sum(L); N
            6-d lattice K+L
            sage: N, N.dual(), latex(N), latex(N.dual())
            (6-d lattice K+L, 6-d lattice K*+L*, K \oplus L, K^* \oplus L^*)

        With default names::

            sage: N = ToricLattice(3).direct_sum(ToricLattice(2))
            sage: N, N.dual(), latex(N), latex(N.dual())
            (5-d lattice N+N, 5-d lattice M+M, N \oplus N, M \oplus M)

        If ``other`` is not a :class:`ToricLattice
        <ToricLatticeFactory>`, fall back to sum of modules::

            sage: ToricLattice(3).direct_sum(ZZ^2)
            Free module of degree 5 and rank 5 over Integer Ring
            Echelon basis matrix:
            [1 0 0 0 0]
            [0 1 0 0 0]
            [0 0 1 0 0]
            [0 0 0 1 0]
            [0 0 0 0 1]
        """
        if not isinstance(other, ToricLattice_generic):
            return super().direct_sum(other)

        def make_name(N1, N2, use_latex=False):
            if use_latex:
                return latex(N1) + r' \oplus ' + latex(N2)
            else:
                return N1._name + '+' + N2._name

        rank = self.rank() + other.rank()
        name = make_name(self, other, False)
        dual_name = make_name(self.dual(), other.dual(), False)
        latex_name = make_name(self, other, True)
        latex_dual_name = make_name(self.dual(), other.dual(), True)
        return ToricLattice(rank, name, dual_name, latex_name, latex_dual_name)

    def intersection(self, other):
        r"""
        Return the intersection of ``self`` and ``other``.

        INPUT:

        - ``other`` -- a toric (sub)lattice.dual

        OUTPUT:

        - a toric (sub)lattice.

        EXAMPLES::

            sage: N = ToricLattice(3)
            sage: Ns1 = N.submodule([N(2,4,0), N(9,12,0)])
            sage: Ns2 = N.submodule([N(1,4,9), N(9,2,0)])
            sage: Ns1.intersection(Ns2)
            Sublattice <N(54, 12, 0)>

        Note that if one of the intersecting sublattices is a sublattice of
        another, no new lattices will be constructed::

            sage: N.intersection(N) is N
            True
            sage: Ns1.intersection(N) is Ns1
            True
            sage: N.intersection(Ns1) is Ns1
            True
        """
        # Lattice-specific input check
        if not isinstance(other, ToricLattice_generic):
            raise TypeError("%s is not a toric lattice!" % other)
        if self.ambient_module() != other.ambient_module():
            raise ValueError("%s and %s have different ambient lattices!" %
                             (self, other))
        # Construct a generic intersection, but make sure to return a lattice.
        I = super().intersection(other)
        if not isinstance(I, ToricLattice_generic):
            I = self.ambient_module().submodule(I.basis())
        return I

    def quotient(self, sub, check=True,
                 positive_point=None, positive_dual_point=None, **kwds):
        """
        Return the quotient of ``self`` by the given sublattice ``sub``.

        INPUT:

        - ``sub`` -- sublattice of self

        - ``check`` -- boolean (default: ``True``); whether or not to check that ``sub`` is
          a valid sublattice

        If the quotient is one-dimensional and torsion free, the
        following two mutually exclusive keyword arguments are also
        allowed. They decide the sign choice for the (single)
        generator of the quotient lattice:

        - ``positive_point`` -- a lattice point of ``self`` not in the
          sublattice ``sub`` (that is, not zero in the quotient
          lattice). The quotient generator will be in the same
          direction as ``positive_point``.

        - ``positive_dual_point`` -- a dual lattice point. The
          quotient generator will be chosen such that its lift has a
          positive product with ``positive_dual_point``. Note: if
          ``positive_dual_point`` is not zero on the sublattice
          ``sub``, then the notion of positivity will depend on the
          choice of lift!

        Further named arguments are passed to the constructor of a toric lattice
        quotient.

        EXAMPLES::

            sage: N = ToricLattice(3)
            sage: Ns = N.submodule([N(2,4,0), N(9,12,0)])
            sage: Q = N/Ns
            sage: Q                                                                     # needs sage.libs.pari
            Quotient with torsion of 3-d lattice N
            by Sublattice <N(1, 8, 0), N(0, 12, 0)>

        Attempting to quotient one lattice by a sublattice of another
        will result in a :exc:`ValueError`::

            sage: N = ToricLattice(3)
            sage: M = ToricLattice(3, name='M')
            sage: Ms = M.submodule([M(2,4,0), M(9,12,0)])
            sage: N.quotient(Ms)
            Traceback (most recent call last):
            ...
            ValueError: M(1, 8, 0) cannot generate a sublattice of
            3-d lattice N

        However, if we forget the sublattice structure, then it is
        possible to quotient by vector spaces or modules constructed
        from any sublattice::

            sage: N = ToricLattice(3)
            sage: M = ToricLattice(3, name='M')
            sage: Ms = M.submodule([M(2,4,0), M(9,12,0)])
            sage: N.quotient(Ms.vector_space())                                         # needs sage.libs.pari
            Quotient with torsion of 3-d lattice N by Sublattice
            <N(1, 8, 0), N(0, 12, 0)>
            sage: N.quotient(Ms.sparse_module())                                        # needs sage.libs.pari
            Quotient with torsion of 3-d lattice N by Sublattice
            <N(1, 8, 0), N(0, 12, 0)>

        See :class:`ToricLattice_quotient` for more examples.

        TESTS:

        We check that :issue:`19603` is fixed::

            sage: # needs sage.geometry.polyhedron
            sage: K = Cone([(1,0,0),(0,1,0)])
            sage: K.lattice()
            3-d lattice N
            sage: K.orthogonal_sublattice()
            Sublattice <M(0, 0, 1)>
            sage: K.lattice().quotient(K.orthogonal_sublattice())
            Traceback (most recent call last):
            ...
            ValueError: M(0, 0, 1) cannot generate a sublattice of
            3-d lattice N

        We can quotient by the trivial sublattice::

            sage: N = ToricLattice(3)
            sage: N.quotient(N.zero_submodule())                                        # needs sage.libs.pari
            3-d lattice, quotient of 3-d lattice N by Sublattice <>

        We can quotient a lattice by itself::

            sage: N = ToricLattice(3)
            sage: N.quotient(N)                                                         # needs sage.libs.pari
            0-d lattice, quotient of 3-d lattice N by Sublattice
            <N(1, 0, 0), N(0, 1, 0), N(0, 0, 1)>
        """
        return ToricLattice_quotient(self, sub, check,
                                     positive_point, positive_dual_point, **kwds)

    def saturation(self):
        r"""
        Return the saturation of ``self``.

        OUTPUT: a :class:`toric lattice <ToricLatticeFactory>`

        EXAMPLES::

            sage: N = ToricLattice(3)
            sage: Ns = N.submodule([(1,2,3), (4,5,6)])
            sage: Ns
            Sublattice <N(1, 2, 3), N(0, 3, 6)>
            sage: Ns_sat = Ns.saturation()
            sage: Ns_sat
            Sublattice <N(1, 0, -1), N(0, 1, 2)>
            sage: Ns_sat is Ns_sat.saturation()
            True
        """
        S = super().saturation()
        return S if isinstance(S, ToricLattice_generic) else self.ambient_module().submodule(S)

    def span(self, gens, base_ring=ZZ, *args, **kwds):
        r"""
        Return the span of the given generators.

        INPUT:

        - ``gens`` -- list of elements of the ambient vector space of
          ``self``

        - ``base_ring`` -- (default: `\ZZ`) base ring for the generated module

        OUTPUT: submodule spanned by ``gens``

        .. NOTE::

            The output need not be a submodule of ``self``, nor even of the
            ambient space. It must, however, be contained in the ambient
            vector space.

        See also :meth:`span_of_basis`,
        :meth:`~sage.modules.free_module.FreeModule_generic_pid.submodule`,
        and
        :meth:`~sage.modules.free_module.FreeModule_generic_pid.submodule_with_basis`,

        EXAMPLES::

            sage: N = ToricLattice(3)
            sage: Ns = N.submodule([N.gen(0)])
            sage: Ns.span([N.gen(1)])
            Sublattice <N(0, 1, 0)>
            sage: Ns.submodule([N.gen(1)])
            Traceback (most recent call last):
            ...
            ArithmeticError: argument gens (= [N(0, 1, 0)]) does not generate a submodule of self
        """
        A = self.ambient_module()
        if base_ring is ZZ and all(g in A for g in gens):
            return ToricLattice_sublattice(A, gens)
        for g in gens:
            if isinstance(g, ToricLatticeElement) and g not in A:
                raise ValueError("%s cannot generate a sublattice of %s"
                                 % (g, A))
        return super().span(gens, base_ring, *args, **kwds)

    def span_of_basis(self, basis, base_ring=ZZ, *args, **kwds):
        r"""
        Return the submodule with the given ``basis``.

        INPUT:

        - ``basis`` -- list of elements of the ambient vector space of
          ``self``

        - ``base_ring`` -- (default: `\ZZ`) base ring for the generated module

        OUTPUT: submodule spanned by ``basis``

        .. NOTE::

            The output need not be a submodule of ``self``, nor even of the
            ambient space. It must, however, be contained in the ambient
            vector space.

        See also :meth:`span`,
        :meth:`~sage.modules.free_module.FreeModule_generic_pid.submodule`,
        and
        :meth:`~sage.modules.free_module.FreeModule_generic_pid.submodule_with_basis`,

        EXAMPLES::

            sage: N = ToricLattice(3)
            sage: Ns = N.span_of_basis([(1,2,3)])
            sage: Ns.span_of_basis([(2,4,0)])
            Sublattice <N(2, 4, 0)>
            sage: Ns.span_of_basis([(1/5,2/5,0), (1/7,1/7,0)])
            Free module of degree 3 and rank 2 over Integer Ring
            User basis matrix:
            [1/5 2/5   0]
            [1/7 1/7   0]

        Of course the input basis vectors must be linearly independent::

            sage: Ns.span_of_basis([(1,2,0), (2,4,0)])
            Traceback (most recent call last):
            ...
            ValueError: the given basis vectors must be linearly independent
        """
        A = self.ambient_module()
        if base_ring is ZZ and all(g in A for g in basis):
            return ToricLattice_sublattice_with_basis(A, basis)
        for g in basis:
            if isinstance(g, ToricLatticeElement) and g not in A:
                raise ValueError("%s cannot generate a sublattice of %s"
                                 % (g, A))
        return super().span_of_basis(basis, base_ring, *args, **kwds)


@richcmp_method
class ToricLattice_ambient(ToricLattice_generic, FreeModule_ambient_pid):
    r"""
    Create a toric lattice.

    See :class:`ToricLattice <ToricLatticeFactory>` for documentation.

    .. WARNING::

        There should be only one toric lattice with the given rank and
        associated names. Using this class directly to create toric lattices
        may lead to unexpected results. Please, use :class:`ToricLattice
        <ToricLatticeFactory>` to create toric lattices.

    TESTS::

        sage: N = ToricLattice(3, "N", "M", "N", "M")
        sage: N
        3-d lattice N
        sage: TestSuite(N).run()
    """

    Element = ToricLatticeElement

    def __init__(self, rank, name, dual_name, latex_name, latex_dual_name):
        r"""
        See :class:`ToricLattice <ToricLatticeFactory>` for documentation.

        TESTS::

            sage: ToricLattice(3, "N", "M", "N", "M")
            3-d lattice N
        """
        super().__init__(ZZ, rank)
        self._name = name
        self._dual_name = dual_name
        self._latex_name = latex_name
        self._latex_dual_name = latex_dual_name

    def _sage_input_(self, sib, coerced):
        r"""
        Return Sage command to reconstruct ``self``.

        See :mod:`sage.misc.sage_input` for details.

        EXAMPLES::

            sage: N = ToricLattice(3, "N", "M", "N", "M")
            sage: sage_input(N, verify=True)
            # Verified
            ToricLattice(3, 'N', 'M')

            sage: N = ToricLattice(3, "N")
            sage: sage_input(N, verify=True)
            # Verified
            ToricLattice(3, 'N', 'N*', 'N', 'N^*')
        """
        args = [self.rank(), self._name, self._dual_name]
        if self._latex_name != self._name or self._latex_dual_name != self._dual_name:
            args.extend([self._latex_name, self._latex_dual_name])
        return sib.name('ToricLattice')(*args)

    def __richcmp__(self, right, op):
        r"""
        Compare ``self`` and ``right``.

        INPUT:

        - ``right`` -- anything

        OUTPUT: boolean

        There is equality if ``right`` is a toric lattice of the same
        dimension as ``self`` and their associated names are the
        same.

        TESTS::

            sage: N3 = ToricLattice(3)
            sage: N4 = ToricLattice(4)
            sage: M3 = N3.dual()
            sage: N3 < N4
            True
            sage: N3 > M3
            True
            sage: N3 == ToricLattice(3)
            True
        """
        if self is right:
            return rich_to_bool(op, 0)
        if type(self) is not type(right):
            return NotImplemented

        lx = self.rank()
        rx = right.rank()
        if lx != rx:
            return richcmp_not_equal(lx, rx, op)
        # If lattices are the same as ZZ-modules, compare associated names
        return richcmp([self._name, self._dual_name,
                        self._latex_name, self._latex_dual_name],
                       [right._name, right._dual_name,
                        right._latex_name, right._latex_dual_name], op)

    def _latex_(self):
        r"""
        Return a LaTeX representation of ``self``.

        OUTPUT: string

        TESTS::

            sage: L = ToricLattice(3, "L")
            sage: L.dual()._latex_()
            'L^*'
        """
        return self._latex_name

    def _repr_(self):
        r"""
        Return a string representation of ``self``.

        OUTPUT: string

        TESTS::

            sage: L = ToricLattice(3, "L")
            sage: L.dual()._repr_()
            '3-d lattice L*'
        """
        return "%d-d lattice %s" % (self.dimension(), self._name)

    def ambient_module(self):
        r"""
        Return the ambient module of ``self``.

        OUTPUT: :class:`toric lattice <ToricLatticeFactory>`

        .. NOTE::

            For any ambient toric lattice its ambient module is the lattice
            itself.

        EXAMPLES::

            sage: N = ToricLattice(3)
            sage: N.ambient_module()
            3-d lattice N
            sage: N.ambient_module() is N
            True
        """
        return self

    def dual(self):
        r"""
        Return the lattice dual to ``self``.

        OUTPUT: :class:`toric lattice <ToricLatticeFactory>`

        EXAMPLES::

            sage: N = ToricLattice(3)
            sage: N
            3-d lattice N
            sage: M = N.dual()
            sage: M
            3-d lattice M
            sage: M.dual() is N
            True

        Elements of dual lattices can act on each other::

            sage: n = N(1,2,3)
            sage: m = M(4,5,6)
            sage: n * m
            32
            sage: m * n
            32
        """
        if "_dual" not in self.__dict__:
            self._dual = ToricLattice(self.rank(), self._dual_name,
                          self._name, self._latex_dual_name, self._latex_name)
        return self._dual

    def plot(self, **options):
        r"""
        Plot ``self``.

        INPUT:

        - any options for toric plots (see :func:`toric_plotter.options
          <sage.geometry.toric_plotter.options>`), none are mandatory.

        OUTPUT: a plot

        EXAMPLES::

            sage: N = ToricLattice(3)
            sage: N.plot()                                                              # needs sage.geometry.polyhedron sage.plot
            Graphics3d Object
        """
        if "show_lattice" not in options:
            # Unless user made an explicit decision, we assume that lattice
            # should be visible no matter what is the size of the bounding box.
            options["show_lattice"] = True
        tp = ToricPlotter(options, self.degree())
        tp.adjust_options()
        return tp.plot_lattice()


class ToricLattice_sublattice_with_basis(ToricLattice_generic,
                                         FreeModule_submodule_with_basis_pid):
    r"""
    Construct the sublattice of ``ambient`` toric lattice with given ``basis``.

    INPUT (same as for
    :class:`~sage.modules.free_module.FreeModule_submodule_with_basis_pid`):

    - ``ambient`` -- ambient :class:`toric lattice <ToricLatticeFactory>` for
      this sublattice

    - ``basis`` -- list of linearly independent elements of ``ambient``, these
      elements will be used as the default basis of the constructed
      sublattice

    - see the base class for other available options

    OUTPUT: sublattice of a toric lattice with a user-specified basis

    See also :class:`ToricLattice_sublattice` if you do not want to specify an
    explicit basis.

    EXAMPLES:

    The intended way to get objects of this class is to use
    :meth:`submodule_with_basis` method of toric lattices::

        sage: N = ToricLattice(3)
        sage: sublattice = N.submodule_with_basis([(1,1,0), (3,2,1)])
        sage: sublattice.has_user_basis()
        True
        sage: sublattice.basis()
        [N(1, 1, 0), N(3, 2, 1)]

    Even if you have provided your own basis, you still can access the
    "standard" one::

        sage: sublattice.echelonized_basis()
        [N(1, 0, 1), N(0, 1, -1)]
    """

    def _repr_(self):
        r"""
        Return a string representation of ``self``.

        OUTPUT: string

        TESTS::

            sage: L = ToricLattice(3, "L")
            sage: L.submodule_with_basis([(3,2,1),(1,2,3)])
            Sublattice <L(3, 2, 1), L(1, 2, 3)>
            sage: print(L.submodule([(3,2,1),(1,2,3)])._repr_())
            Sublattice <L(1, 2, 3), L(0, 4, 8)>
        """
        s = 'Sublattice '
        s += '<'
        s += ', '.join(map(str, self.basis()))
        s += '>'
        return s

    def _latex_(self):
        r"""
        Return a LaTeX representation of ``self``.

        OUTPUT: string

        TESTS::

            sage: L = ToricLattice(3, "L")
            sage: L.submodule_with_basis([(3,2,1),(1,2,3)])._latex_()
            '\\left\\langle\\left(3,\\,2,\\,1\\right)_{L},
             \\left(1,\\,2,\\,3\\right)_{L}\\right\\rangle'
            sage: L.submodule([(3,2,1),(1,2,3)])._latex_()
            '\\left\\langle\\left(1,\\,2,\\,3\\right)_{L},
             \\left(0,\\,4,\\,8\\right)_{L}\\right\\rangle'
        """
        s = '\\left\\langle'
        s += ', '.join(b._latex_() for b in self.basis())
        s += '\\right\\rangle'
        return s

    def dual(self):
        r"""
        Return the lattice dual to ``self``.

        OUTPUT: a :class:`toric lattice quotient <ToricLattice_quotient>`

        EXAMPLES::

            sage: N = ToricLattice(3)
            sage: Ns = N.submodule([(1,1,0), (3,2,1)])
            sage: Ns.dual()                                                             # needs sage.libs.pari
            2-d lattice, quotient of 3-d lattice M by Sublattice <M(1, -1, -1)>
        """
        if "_dual" not in self.__dict__:
            if self is not self.saturation():
                raise ValueError("only dual lattices of saturated sublattices "
                                 "can be constructed! Got %s." % self)
            self._dual = (self.ambient_module().dual() /
                          self.basis_matrix().transpose().integer_kernel())
            self._dual._dual = self
        return self._dual

    def plot(self, **options):
        r"""
        Plot ``self``.

        INPUT:

        - any options for toric plots (see :func:`toric_plotter.options
          <sage.geometry.toric_plotter.options>`), none are mandatory.

        OUTPUT: a plot

        EXAMPLES::

            sage: N = ToricLattice(3)
            sage: sublattice = N.submodule_with_basis([(1,1,0), (3,2,1)])
            sage: sublattice.plot()                                                     # needs sage.geometry.polyhedron sage.plot
            Graphics3d Object

        Now we plot both the ambient lattice and its sublattice::

            sage: N.plot() + sublattice.plot(point_color='red')                         # needs sage.geometry.polyhedron sage.plot
            Graphics3d Object
        """
        if "show_lattice" not in options:
            # Unless user made an explicit decision, we assume that lattice
            # should be visible no matter what is the size of the bounding box.
            options["show_lattice"] = True
        if "lattice_filter" in options:
            old = options["lattice_filter"]
            options["lattice_filter"] = lambda pt: pt in self and old(pt)
        else:
            options["lattice_filter"] = lambda pt: pt in self
        tp = ToricPlotter(options, self.degree())
        tp.adjust_options()
        return tp.plot_lattice()


class ToricLattice_sublattice(ToricLattice_sublattice_with_basis,
                              FreeModule_submodule_pid):
    r"""
    Construct the sublattice of ``ambient`` toric lattice generated by ``gens``.

    INPUT (same as for
    :class:`~sage.modules.free_module.FreeModule_submodule_pid`):

    - ``ambient`` -- ambient :class:`toric lattice <ToricLatticeFactory>` for
      this sublattice

    - ``gens`` -- list of elements of ``ambient`` generating the constructed
      sublattice

    - see the base class for other available options

    OUTPUT: sublattice of a toric lattice with an automatically chosen basis

    See also :class:`ToricLattice_sublattice_with_basis` if you want to
    specify an explicit basis.

    EXAMPLES:

    The intended way to get objects of this class is to use
    :meth:`submodule` method of toric lattices::

        sage: N = ToricLattice(3)
        sage: sublattice = N.submodule([(1,1,0), (3,2,1)])
        sage: sublattice.has_user_basis()
        False
        sage: sublattice.basis()
        [N(1, 0, 1), N(0, 1, -1)]

    For sublattices without user-specified basis, the basis obtained above is
    the same as the "standard" one::

        sage: sublattice.echelonized_basis()
        [N(1, 0, 1), N(0, 1, -1)]
    """
    pass


class ToricLattice_quotient_element(FGP_Element):
    r"""
    Create an element of a toric lattice quotient.

    .. WARNING::

        You probably should not construct such elements explicitly.

    INPUT:

    - same as for :class:`~sage.modules.fg_pid.fgp_element.FGP_Element`.

    OUTPUT: element of a toric lattice quotient

    TESTS::

        sage: N = ToricLattice(3)
        sage: sublattice = N.submodule([(1,1,0), (3,2,1)])
        sage: Q = N/sublattice
        sage: e = Q(1,2,3)
        sage: e
        N[1, 2, 3]
        sage: e2 = Q(N(2,3,3))
        sage: e2
        N[2, 3, 3]
        sage: e == e2                                                                   # needs sage.libs.pari
        True
        sage: e.vector()                                                                # needs sage.libs.pari
        (-4)
        sage: e2.vector()                                                               # needs sage.libs.pari
        (-4)
    """

    def _latex_(self):
        r"""
        Return a LaTeX representation of ``self``.

        OUTPUT: string

        TESTS::

            sage: N = ToricLattice(3)
            sage: Ns = N.submodule([N(2,4,0), N(9,12,0)])
            sage: Q = N/Ns
            sage: print(Q.gen(0)._latex_())                                             # needs sage.libs.pari
            \left[0,\,1,\,0\right]_{N}
        """
        return latex(self.lift()).replace("(", "[", 1).replace(")", "]", 1)

    def _repr_(self):
        r"""
        Return a string representation of ``self``.

        OUTPUT: string

        TESTS::

            sage: N = ToricLattice(3)
            sage: Ns = N.submodule([N(2,4,0), N(9,12,0)])
            sage: Q = N/Ns
            sage: print(Q.gen(0)._repr_())                                              # needs sage.libs.pari
            N[0, 1, 0]
        """
        return str(self.lift()).replace("(", "[", 1).replace(")", "]", 1)

    def set_immutable(self):
        r"""
        Make ``self`` immutable.

        OUTPUT: none

        .. NOTE:: Elements of toric lattice quotients are always immutable, so
            this method does nothing, it is introduced for compatibility
            purposes only.

        EXAMPLES::

            sage: N = ToricLattice(3)
            sage: Ns = N.submodule([N(2,4,0), N(9,12,0)])
            sage: Q = N/Ns
            sage: Q.0.set_immutable()                                                   # needs sage.libs.pari
        """
        pass


class ToricLattice_quotient(FGP_Module_class):
    r"""
    Construct the quotient of a toric lattice ``V`` by its sublattice ``W``.

    INPUT:

    - ``V`` -- ambient toric lattice

    - ``W`` -- sublattice of ``V``

    - ``check`` -- boolean (default: ``True``); whether to check correctness of input
      or not

    If the quotient is one-dimensional and torsion free, the following
    two mutually exclusive keyword arguments are also allowed. They
    decide the sign choice for the (single) generator of the quotient
    lattice:

    - ``positive_point`` -- a lattice point of ``self`` not in the
      sublattice ``sub`` (that is, not zero in the quotient
      lattice). The quotient generator will be in the same direction
      as ``positive_point``.

    - ``positive_dual_point`` -- a dual lattice point. The quotient
      generator will be chosen such that its lift has a positive
      product with ``positive_dual_point``. Note: if
      ``positive_dual_point`` is not zero on the sublattice ``sub``,
      then the notion of positivity will depend on the choice of lift!

    Further given named arguments are passed to the constructor of an FGP
    module.

    OUTPUT: quotient of ``V`` by ``W``

    EXAMPLES:

    The intended way to get objects of this class is to use
    :meth:`quotient` method of toric lattices::

        sage: N = ToricLattice(3)
        sage: sublattice = N.submodule([(1,1,0), (3,2,1)])
        sage: Q = N/sublattice
        sage: Q                                                                         # needs sage.libs.pari
        1-d lattice, quotient of 3-d lattice N by Sublattice <N(1, 0, 1), N(0, 1, -1)>
        sage: Q.gens()                                                                  # needs sage.libs.pari
        (N[1, 0, 0],)

    Here, ``sublattice`` happens to be of codimension one in ``N``. If
    you want to prescribe the sign of the quotient generator, you can
    do either::

        sage: Q = N.quotient(sublattice, positive_point=N(0,0,-1)); Q                   # needs sage.libs.pari
        1-d lattice, quotient of 3-d lattice N by Sublattice <N(1, 0, 1), N(0, 1, -1)>
        sage: Q.gens()                                                                  # needs sage.libs.pari
        (N[1, 0, 0],)

    or::

        sage: M = N.dual()
        sage: Q = N.quotient(sublattice, positive_dual_point=M(1,0,0)); Q               # needs sage.libs.pari
        1-d lattice, quotient of 3-d lattice N by Sublattice <N(1, 0, 1), N(0, 1, -1)>
        sage: Q.gens()                                                                  # needs sage.libs.pari
        (N[1, 0, 0],)

    TESTS::

        sage: loads(dumps(Q)) == Q
        True
        sage: loads(dumps(Q)).gens() == Q.gens()                                        # needs sage.libs.pari
        True
    """

    def __init__(self, V, W, check=True, positive_point=None, positive_dual_point=None, **kwds):
        r"""
        The constructor.

        See :class:`ToricLattice_quotient` for an explanation of the arguments.

        EXAMPLES::

            sage: N = ToricLattice(3)
            sage: from sage.geometry.toric_lattice import ToricLattice_quotient
            sage: ToricLattice_quotient(N, N.span([N(1,2,3)]))                          # needs sage.libs.pari
            2-d lattice, quotient of 3-d lattice N by Sublattice <N(1, 2, 3)>

        An :exc:`ArithmeticError` will be raised if ``W`` is not a
        sublattice of ``V``::

            sage: N = ToricLattice(3)
            sage: Ns = N.submodule([N.gen(0)])
            sage: Ns
            Sublattice <N(1, 0, 0)>
            sage: Ns.span([N.gen(1)])
            Sublattice <N(0, 1, 0)>
            sage: Ns.quotient(Ns.span([N.gen(1)]))                                      # needs sage.libs.pari
            Traceback (most recent call last):
            ...
            ArithmeticError: W must be a sublattice of V
        """
        if check:
            try:
                W = V.submodule(W)
            except (TypeError, ArithmeticError):
                raise ArithmeticError("W must be a sublattice of V")
        super().__init__(V, W, check, **kwds)
        if (positive_point, positive_dual_point) == (None, None):
            self._flip_sign_of_generator = False
            return

        self._flip_sign_of_generator = False
        assert self.is_torsion_free() and self.ngens() == 1, \
            'You may only specify a positive direction in the codimension one case.'
        quotient_generator = self.gen(0)
        lattice = self.V().ambient_module()
        if (positive_point is not None) and (positive_dual_point is None):
            assert positive_point in lattice, 'positive_point must be a lattice point.'
            point_quotient = self(positive_point)
            scalar_product = quotient_generator.vector()[0] * point_quotient.vector()[0]
            if scalar_product == 0:
                raise ValueError(str(positive_point)+' is zero in the quotient.')
        elif (positive_point is None) and (positive_dual_point is not None):
            assert positive_dual_point in lattice.dual(), 'positive_dual_point must be a dual lattice point.'
            scalar_product = quotient_generator.lift() * positive_dual_point
            if scalar_product == 0:
                raise ValueError(str(positive_dual_point)+' is zero on the lift of the quotient generator.')
        else:
            raise ValueError('You may not specify both positive_point and positive_dual_point.')
        self._flip_sign_of_generator = (scalar_product < 0)

    def gens(self) -> tuple:
        """
        Return the generators of the quotient.

        OUTPUT:

        A tuple of :class:`ToricLattice_quotient_element` generating
        the quotient.

        EXAMPLES::

            sage: N = ToricLattice(3)
            sage: Q = N.quotient(N.span([N(1,2,3), N(0,2,1)]), positive_point=N(0,-1,0))    # needs sage.libs.pari
            sage: Q.gens()                                                                  # needs sage.libs.pari
            (N[0, -1, 0],)
        """
        gens = self.smith_form_gens()
        if self._flip_sign_of_generator:
            assert len(gens) == 1
            return (-gens[0],)
        else:
            return gens

    # Should be overridden in derived classes.
    Element = ToricLattice_quotient_element

    def _element_constructor_(self, *x, **kwds):
        r"""
        Construct an element of ``self``.

        INPUT:

        - element of a compatible toric object (lattice, sublattice, quotient)
          or something that defines such an element (list, generic vector,
          etc.).

        OUTPUT:

        - :class:`toric lattice quotient element
          <ToricLattice_quotient_element>`.

        EXAMPLES::

            sage: N = ToricLattice(3)
            sage: Ns = N.submodule([N(2,4,0), N(9,12,0)])
            sage: Q = N/Ns
            sage: x = Q(1,2,3)  # indirect doctest
            sage: x
            N[1, 2, 3]
            sage: type(x)
            <class 'sage.geometry.toric_lattice.ToricLattice_quotient_with_category.element_class'>
            sage: x is Q(x)
            True
            sage: x.parent() is Q
            True
            sage: x == Q(N(1,2,3))                                                          # needs sage.libs.pari
            True
            sage: y = Q(3,6,3)
            sage: y
            N[3, 6, 3]
            sage: x == y                                                                    # needs sage.libs.pari
            True
        """
        if len(x) == 1 and (x[0] not in ZZ or x[0] == 0):
            x = x[0]
        if parent(x) is self:
            return x
        try:
            x = x.lift()
        except AttributeError:
            pass
        try:
            return self.element_class(self, self._V(x), **kwds)
        except TypeError:
            return self.linear_combination_of_smith_form_gens(x)

    def _latex_(self):
        r"""
        Return a LaTeX representation of ``self``.

        OUTPUT: string

        TESTS::

            sage: N = ToricLattice(3)
            sage: Ns = N.submodule([N(2,4,0), N(9,12,0)])
            sage: Q = N/Ns
            sage: print(Q._latex_())
            N / \left\langle\left(1,\,8,\,0\right)_{N}, \left(0,\,12,\,0\right)_{N}\right\rangle
            sage: Ns = N.submodule([N(1,4,0)])
            sage: Q = N/Ns
            sage: print(Q._latex_())
            N / \left\langle\left(1,\,4,\,0\right)_{N}\right\rangle
        """
        return "%s / %s" % (latex(self.V()), latex(self.W()))

    def _repr_(self):
        r"""
        Return a string representation of ``self``.

        OUTPUT: string

        TESTS::

            sage: N = ToricLattice(3)
            sage: Ns = N.submodule([N(2,4,0), N(9,12,0)])
            sage: Q = N/Ns
            sage: print(Q._repr_())                                                         # needs sage.libs.pari
            Quotient with torsion of 3-d lattice N
            by Sublattice <N(1, 8, 0), N(0, 12, 0)>
            sage: Ns = N.submodule([N(1,4,0)])
            sage: Q = N/Ns
            sage: print(Q._repr_())                                                         # needs sage.libs.pari
            2-d lattice, quotient of 3-d lattice N
            by Sublattice <N(1, 4, 0)>
        """
        if self.is_torsion_free():
            return "%d-d lattice, quotient of %s by %s" % (self.rank(),
                                                           self.V(), self.W())
        else:
            return "Quotient with torsion of %s by %s" % (self.V(), self.W())

    def _module_constructor(self, V, W, check=True):
        r"""
        Construct new quotient modules.

        INPUT:

        - ``V`` -- ambient toric lattice

        - ``W`` -- sublattice of ``V``

        - ``check`` -- boolean (default: ``True``); whether to check
          correctness of input or not

        TESTS::

            sage: N = ToricLattice(3)
            sage: Ns = N.submodule([N(2,4,0), N(9,12,0)])
            sage: Q = N/Ns; Q                                                               # needs sage.libs.pari
            Quotient with torsion of 3-d lattice N by Sublattice <N(1, 8, 0), N(0, 12, 0)>
            sage: Q._module_constructor(N,Ns)                                               # needs sage.libs.pari
            Quotient with torsion of 3-d lattice N by Sublattice <N(1, 8, 0), N(0, 12, 0)>
        """
        return ToricLattice_quotient(V,W,check)

    def base_extend(self, R):
        r"""
        Return the base change of ``self`` to the ring ``R``.

        INPUT:

        - ``R`` -- either `\ZZ` or `\QQ`

        OUTPUT: ``self`` if `R=\ZZ`, quotient of the base extension of the ambient
        lattice by the base extension of the sublattice if `R=\QQ`

        EXAMPLES::

            sage: N = ToricLattice(3)
            sage: Ns = N.submodule([N(2,4,0), N(9,12,0)])
            sage: Q = N/Ns
            sage: Q.base_extend(ZZ) is Q
            True
            sage: Q.base_extend(QQ)
            Vector space quotient V/W of dimension 1 over Rational Field where
            V: Vector space of dimension 3 over Rational Field
            W: Vector space of degree 3 and dimension 2 over Rational Field
            Basis matrix:
            [1 0 0]
            [0 1 0]
        """
        if R is ZZ:
            return self
        if R is QQ:
            return self.V().base_extend(R) / self.W().base_extend(R)
        raise NotImplementedError("quotients of toric lattices can only be "
                                  "extended to ZZ or QQ, not %s!" % R)

    def is_torsion_free(self):
        r"""
        Check if ``self`` is torsion-free.

        OUTPUT: ``True`` if ``self`` has no torsion and ``False`` otherwise

        EXAMPLES::

            sage: N = ToricLattice(3)
            sage: Ns = N.submodule([N(2,4,0), N(9,12,0)])
            sage: Q = N/Ns
            sage: Q.is_torsion_free()                                                       # needs sage.libs.pari
            False
            sage: Ns = N.submodule([N(1,4,0)])
            sage: Q = N/Ns
            sage: Q.is_torsion_free()                                                       # needs sage.libs.pari
            True
        """
        return sum(self.invariants()) == 0

    def dual(self):
        r"""
        Return the lattice dual to ``self``.

        OUTPUT: a :class:`toric lattice quotient <ToricLattice_quotient>`

        EXAMPLES::

            sage: N = ToricLattice(3)
            sage: Ns = N.submodule([(1, -1, -1)])
            sage: Q = N / Ns
            sage: Q.dual()
            Sublattice <M(1, 0, 1), M(0, 1, -1)>
        """
        if "_dual" not in self.__dict__:
            self._dual = self.V().dual().submodule(
                    self.W().basis_matrix().transpose().integer_kernel().gens())
            self._dual._dual = self
        return self._dual

    def rank(self):
        r"""
        Return the rank of ``self``.

        OUTPUT: integer; the dimension of the free part of the quotient

        EXAMPLES::

            sage: N = ToricLattice(3)
            sage: Ns = N.submodule([N(2,4,0), N(9,12,0)])
            sage: Q = N/Ns
            sage: Q.ngens()                                                                 # needs sage.libs.pari
            2
            sage: Q.rank()
            1
            sage: Ns = N.submodule([N(1,4,0)])
            sage: Q = N/Ns
            sage: Q.ngens()                                                                 # needs sage.libs.pari
            2
            sage: Q.rank()
            2
        """
        return self.V().rank() - self.W().rank()

    dimension = rank

    def coordinate_vector(self, x, reduce=False):
        """
        Return coordinates of ``x`` with respect to the optimized
        representation of ``self``.

        INPUT:

        - ``x`` -- element of ``self`` or convertible to ``self``

        - ``reduce`` -- (default: ``False``) if ``True``, reduce coefficients
          modulo invariants

        OUTPUT: the coordinates as a vector

        EXAMPLES::

            sage: # needs sage.libs.pari
            sage: N = ToricLattice(3)
            sage: Q = N.quotient(N.span([N(1,2,3), N(0,2,1)]), positive_point=N(0,-1,0))
            sage: q = Q.gen(0); q
            N[0, -1, 0]
            sage: q.vector()  # indirect test
            (1)
            sage: Q.coordinate_vector(q)
            (1)
        """
        coordinates = super().coordinate_vector(x, reduce)
        if self._flip_sign_of_generator:
            assert len(coordinates) == 1, "Sign flipped for a multi-dimensional quotient!"
            return -coordinates
        else:
            return coordinates



# sage_setup: distribution = sagemath-modules
r"""
Advanced matroid functionality

This module collects a number of advanced functions which are not directly
available to the end user by default. To import them into the main namespace,
type::

    sage: from sage.matroids.advanced import *

This adds the following to the main namespace:

    - Matroid classes:

        - :class:`BasisMatroid <sage.matroids.basis_matroid.BasisMatroid>`
        - :class:`CircuitsMatroid <sage.matroids.circuits_matroid.CircuitsMatroid>`
        - :class:`CircuitClosuresMatroid <sage.matroids.circuit_closures_matroid.CircuitClosuresMatroid>`
        - :class:`DualMatroid <sage.matroids.dual_matroid.DualMatroid>`
        - :class:`FlatsMatroid <sage.matroids.flats_matroid.FlatsMatroid>`
        - :class:`GraphicMatroid <sage.matroids.graphic_matroid.GraphicMatroid>`
        - :class:`LinearMatroid <sage.matroids.linear_matroid.LinearMatroid>`
        - :class:`RegularMatroid <sage.matroids.linear_matroid.RegularMatroid>`
        - :class:`BinaryMatroid <sage.matroids.linear_matroid.BinaryMatroid>`
        - :class:`TernaryMatroid <sage.matroids.linear_matroid.TernaryMatroid>`
        - :class:`QuaternaryMatroid <sage.matroids.linear_matroid.QuaternaryMatroid>`
        - :class:`MinorMatroid <sage.matroids.minor_matroid.MinorMatroid>`
        - :class:`RankMatroid <sage.matroids.rank_matroid.RankMatroid>`

    Note that you can construct all of these through the
    :func:`Matroid() <sage.matroids.constructor.Matroid>` function, which is
    available on startup. Using the classes directly can sometimes be useful
    for faster code (e.g. if your code calls ``Matroid()`` frequently).

    - Other classes:

        - :class:`LinearSubclasses <sage.matroids.extension.LinearSubclasses>`
        - :class:`MatroidExtensions <sage.matroids.extension.MatroidExtensions>`

    Instances of these classes are returned by the methods
    :meth:`Matroid.linear_subclasses() <sage.matroids.matroid.Matroid.linear_subclasses>`
    and
    :meth:`Matroid.extensions() <sage.matroids.matroid.Matroid.extensions>`.

    - Useful functions:

        - :func:`setprint() <sage.matroids.utilities.setprint>`
        - :func:`newlabel() <sage.matroids.utilities.newlabel>`
        - :func:`get_nonisomorphic_matroids() <sage.matroids.utilities.get_nonisomorphic_matroids>`
        - :func:`lift_cross_ratios() <sage.matroids.utilities.lift_cross_ratios>`
        - :func:`lift_map() <sage.matroids.utilities.lift_map>`
        - :func:`cmp_elements_key() <sage.matroids.utilities.cmp_elements_key>`

AUTHORS:

- Stefan van Zwam (2013-04-01): initial version
"""

from sage.matroids import matroid, basis_exchange_matroid, lean_matrix

from sage.matroids.basis_matroid import BasisMatroid
from sage.matroids.circuits_matroid import CircuitsMatroid
from sage.matroids.circuit_closures_matroid import CircuitClosuresMatroid
from sage.matroids.dual_matroid import DualMatroid
from sage.matroids.linear_matroid import LinearMatroid, RegularMatroid, BinaryMatroid, TernaryMatroid, QuaternaryMatroid
from sage.matroids.minor_matroid import MinorMatroid
from sage.matroids.rank_matroid import RankMatroid
from sage.matroids.union_matroid import MatroidUnion, MatroidSum, PartitionMatroid

# lazy import of GraphicMatroid and FlatsMatroid for modularization purposes
from sage.misc.lazy_import import lazy_import
lazy_import('sage.matroids.graphic_matroid', 'GraphicMatroid')
lazy_import('sage.matroids.flats_matroid', 'FlatsMatroid')

from sage.matroids.extension import LinearSubclasses, MatroidExtensions
from sage.matroids.utilities import setprint, newlabel, get_nonisomorphic_matroids, lift_cross_ratios, lift_map, cmp_elements_key

del lazy_import



# sage_setup: distribution = sagemath-modules
r"""
Base classes for reflexive modules
"""

from sage.misc.abstract_method import abstract_method
from sage.structure.parent import Parent


class ReflexiveModule_abstract(Parent):
    r"""
    Abstract base class for reflexive modules.

    An `R`-module `M` is *reflexive* if the natural map from `M` to its double
    dual `M^{**}` is an isomorphism.

    In the category of `R`-modules, the dual module `M^*` is
    the `R`-module of linear functionals `\phi:\ M \longrightarrow R`.
    However, we do not make the assumption that the dual module
    (obtained by :meth:`dual`) is in the category :class:`Homsets`.

    We identify the double dual `M^{**}` with `M`.

    Tensor products of reflexive modules are reflexive. We identify all
    tensor products of `k` copies of `M` and `l` copies of `M^*` and
    denote it by `T^{(k,l)}(M)`. The :meth:`tensor_type` of such a tensor
    product is the pair `(k, l)`, and `M` is called its :meth:`base_module`.

    There are three abstract subclasses:

    - :class:`ReflexiveModule_base` is the base class for implementations
      of base modules `M`.

    - :class:`ReflexiveModule_dual` is the base class for implementations
      of duals `M^*`.

    - :class:`ReflexiveModule_tensor` is the base class for implementations
      of tensor modules `T^{(k,l)}(M)`.

    TESTS::

        sage: from sage.tensor.modules.reflexive_module import (
        ....:     ReflexiveModule_abstract, ReflexiveModule_base,
        ....:     ReflexiveModule_dual, ReflexiveModule_tensor)
        sage: M = FiniteRankFreeModule(ZZ, 3)
        sage: isinstance(M, ReflexiveModule_abstract)
        True
        sage: isinstance(M, ReflexiveModule_base)
        True
        sage: isinstance(M.dual(), ReflexiveModule_abstract)
        True
        sage: isinstance(M.dual(), ReflexiveModule_dual)
        True
        sage: isinstance(M.tensor_module(1, 1), ReflexiveModule_abstract)
        True
        sage: isinstance(M.tensor_module(1, 1), ReflexiveModule_tensor)
        True
    """

    @abstract_method(optional=True)
    def tensor_type(self):
        r"""
        Return the tensor type of ``self``.

        OUTPUT:

        - pair `(k,l)` such that ``self`` is the module tensor product
          `T^{(k,l)}(M)`, where `M` is the :meth:`base_module` of ``self``.

        EXAMPLES::

            sage: M = FiniteRankFreeModule(ZZ, 3)
            sage: T = M.tensor_module(1, 2)
            sage: T.tensor_type()
            (1, 2)
        """

    @abstract_method
    def base_module(self):
        r"""
        Return the module on which ``self`` is constructed.

        EXAMPLES::

            sage: M = FiniteRankFreeModule(ZZ, 3)
            sage: M.base_module() is M
            True
            sage: M.dual().base_module() is M
            True
            sage: M.tensor_module(1, 2).base_module() is M
            True
        """

    def dual(self):
        r"""
        Return the dual module.

        EXAMPLES::

            sage: M = FiniteRankFreeModule(ZZ, 3)
            sage: M.dual()
            Dual of the Rank-3 free module over the Integer Ring
            sage: M.dual().dual()
            Rank-3 free module over the Integer Ring
            sage: M.tensor_module(1, 2)
            Free module of type-(1,2) tensors on the Rank-3 free module over the Integer Ring
            sage: M.tensor_module(1, 2).dual()
            Free module of type-(2,1) tensors on the Rank-3 free module over the Integer Ring
        """
        k, l = self.tensor_type()
        return self.base_module().tensor_module(l, k)

    def tensor(self, *args, **kwds):
        # Until https://github.com/sagemath/sage/issues/30373 is done,
        # TensorProductFunctor._functor_name is "tensor", so here we delegate.
        r"""
        Return the tensor product of ``self`` and ``others``.

        This method is invoked when :class:`~sage.categories.tensor.TensorProductFunctor`
        is applied to parents.

        It just delegates to :meth:`tensor_product`.

        EXAMPLES::

            sage: M = FiniteRankFreeModule(QQ, 2); M
            2-dimensional vector space over the Rational Field
            sage: M20 = M.tensor_module(2, 0); M20
            Free module of type-(2,0) tensors on the 2-dimensional vector space over the Rational Field
            sage: tensor([M20, M20])
            Free module of type-(4,0) tensors on the 2-dimensional vector space over the Rational Field
        """
        return self.tensor_product(*args, **kwds)

    def tensor_power(self, n):
        r"""
        Return the ``n``-fold tensor product of ``self``.

        EXAMPLES::

            sage: M = FiniteRankFreeModule(QQ, 2)
            sage: M.tensor_power(3)
            Free module of type-(3,0) tensors on the 2-dimensional vector space over the Rational Field
            sage: M.tensor_module(1,2).tensor_power(3)
            Free module of type-(3,6) tensors on the 2-dimensional vector space over the Rational Field
        """
        tensor_type = self.tensor_type()
        return self.base_module().tensor_module(n * tensor_type[0], n * tensor_type[1])

    def tensor_product(self, *others):
        r"""
        Return the tensor product of ``self`` and ``others``.

        EXAMPLES::

            sage: M = FiniteRankFreeModule(QQ, 2)
            sage: M.tensor_product(M)
            Free module of type-(2,0) tensors on the 2-dimensional vector space over the Rational Field
            sage: M.tensor_product(M.dual())
            Free module of type-(1,1) tensors on the 2-dimensional vector space over the Rational Field
            sage: M.dual().tensor_product(M, M.dual())
            Free module of type-(1,2) tensors on the 2-dimensional vector space over the Rational Field
            sage: M.tensor_product(M.tensor_module(1,2))
            Free module of type-(2,2) tensors on the 2-dimensional vector space over the Rational Field
            sage: M.tensor_module(1,2).tensor_product(M)
            Free module of type-(2,2) tensors on the 2-dimensional vector space over the Rational Field
            sage: M.tensor_module(1,1).tensor_product(M.tensor_module(1,2))
            Free module of type-(2,3) tensors on the 2-dimensional vector space over the Rational Field

            sage: Sym2M = M.tensor_module(2, 0, sym=range(2)); Sym2M
            Free module of fully symmetric type-(2,0) tensors on the 2-dimensional vector space over the Rational Field
            sage: Sym01x23M = Sym2M.tensor_product(Sym2M); Sym01x23M
            Free module of type-(4,0) tensors on the 2-dimensional vector space over the Rational Field,
             with symmetry on the index positions (0, 1), with symmetry on the index positions (2, 3)
            sage: Sym01x23M._index_maps
            ((0, 1), (2, 3))

            sage: N = M.tensor_module(3, 3, sym=[1, 2], antisym=[3, 4]); N
            Free module of type-(3,3) tensors on the 2-dimensional vector space over the Rational Field,
             with symmetry on the index positions (1, 2),
             with antisymmetry on the index positions (3, 4)
            sage: NxN = N.tensor_product(N); NxN
            Free module of type-(6,6) tensors on the 2-dimensional vector space over the Rational Field,
             with symmetry on the index positions (1, 2), with symmetry on the index positions (4, 5),
             with antisymmetry on the index positions (6, 7), with antisymmetry on the index positions (9, 10)
            sage: NxN._index_maps
            ((0, 1, 2, 6, 7, 8), (3, 4, 5, 9, 10, 11))
        """
        from sage.modules.free_module_element import vector
        from .comp import CompFullySym, CompFullyAntiSym, CompWithSym

        base_module = self.base_module()
        if not all(module.base_module() == base_module for module in others):
            raise NotImplementedError('all factors must be tensor modules over the same base module')
        factors = [self] + list(others)
        result_tensor_type = sum(vector(factor.tensor_type()) for factor in factors)
        result_sym = []
        result_antisym = []
        # Keep track of reordering of the contravariant and covariant indices
        # (compatible with FreeModuleTensor.__mul__)
        index_maps = []
        running_indices = vector([0, result_tensor_type[0]])
        for factor in factors:
            tensor_type = factor.tensor_type()
            index_map = tuple(i + running_indices[0] for i in range(tensor_type[0]))
            index_map += tuple(i + running_indices[1] for i in range(tensor_type[1]))
            index_maps.append(index_map)

            if tensor_type[0] + tensor_type[1] > 1:
                basis_sym = factor._basis_sym()
                all_indices = tuple(range(tensor_type[0] + tensor_type[1]))
                if isinstance(basis_sym, CompFullySym):
                    sym = [all_indices]
                    antisym = []
                elif isinstance(basis_sym, CompFullyAntiSym):
                    sym = []
                    antisym = [all_indices]
                elif isinstance(basis_sym, CompWithSym):
                    sym = basis_sym._sym
                    antisym = basis_sym._antisym
                else:
                    sym = antisym = []

                def map_isym(isym):
                    return tuple(index_map[i] for i in isym)

                result_sym.extend(tuple(index_map[i] for i in isym) for isym in sym)
                result_antisym.extend(tuple(index_map[i] for i in isym) for isym in antisym)

            running_indices += vector(tensor_type)

        result = base_module.tensor_module(*result_tensor_type,
                                           sym=result_sym, antisym=result_antisym)
        result._index_maps = tuple(index_maps)
        return result


class ReflexiveModule_base(ReflexiveModule_abstract):
    r"""
    Abstract base class for reflexive modules that are base modules.

    TESTS::

        sage: from sage.tensor.modules.reflexive_module import ReflexiveModule_base
        sage: M = FiniteRankFreeModule(ZZ, 3, name='M')
        sage: isinstance(M, ReflexiveModule_base)
        True
    """

    def base_module(self):
        r"""
        Return the free module on which ``self`` is constructed, namely ``self`` itself.

        EXAMPLES::

            sage: M = FiniteRankFreeModule(ZZ, 3, name='M')
            sage: M.base_module() is M
            True

            sage: M = Manifold(2, 'M')                                                  # needs sage.symbolic
            sage: XM = M.vector_field_module()                                          # needs sage.symbolic
            sage: XM.base_module() is XM                                                # needs sage.symbolic
            True
        """
        return self

    def tensor_type(self):
        r"""
        Return the tensor type of ``self``, the pair `(1, 0)`.

        EXAMPLES::

            sage: M = FiniteRankFreeModule(ZZ, 3)
            sage: M.tensor_type()
            (1, 0)

            sage: M = Manifold(2, 'M')                                                  # needs sage.symbolic
            sage: XM = M.vector_field_module()                                          # needs sage.symbolic
            sage: XM.tensor_type()                                                      # needs sage.symbolic
            (1, 0)
        """
        return (1, 0)

    def dual(self):
        r"""
        Return the dual module.

        EXAMPLES::

            sage: M = FiniteRankFreeModule(ZZ, 3, name='M')
            sage: M.dual()
            Dual of the Rank-3 free module M over the Integer Ring
        """
        return self.tensor_module(0, 1)

    @abstract_method
    def tensor_module(self, k, l, **kwds):
        r"""
        Return the module of all tensors of type `(k, l)` defined on ``self``.

        EXAMPLES::

            sage: M = FiniteRankFreeModule(ZZ, 3)
            sage: M.tensor_module(1, 2)
            Free module of type-(1,2) tensors on the Rank-3 free module over the Integer Ring
        """


class ReflexiveModule_dual(ReflexiveModule_abstract):
    r"""
    Abstract base class for reflexive modules that are the duals of base modules.

    TESTS::

        sage: from sage.tensor.modules.reflexive_module import ReflexiveModule_dual
        sage: M = FiniteRankFreeModule(ZZ, 3, name='M')
        sage: isinstance(M.dual(), ReflexiveModule_dual)
        True
    """

    def tensor_type(self):
        r"""
        Return the tensor type of ``self``.

        EXAMPLES::

            sage: M = FiniteRankFreeModule(ZZ, 3, name='M')
            sage: M.dual().tensor_type()
            (0, 1)
        """
        return (0, 1)

    def construction(self):
        r"""
        Return the functorial construction of ``self``.

        This implementation just returns ``None``, as no functorial construction is implemented.

        TESTS::

            sage: M = FiniteRankFreeModule(ZZ, 3, name='M')
            sage: e = M.basis('e')
            sage: A = M.dual()
            sage: A.construction() is None
            True
        """
        # Until https://github.com/sagemath/sage/issues/34605 is done
        return None


class ReflexiveModule_tensor(ReflexiveModule_abstract):
    r"""
    Abstract base class for reflexive modules that are tensor products of base modules.

    TESTS::

        sage: from sage.tensor.modules.reflexive_module import ReflexiveModule_tensor
        sage: M = FiniteRankFreeModule(ZZ, 3, name='M')
        sage: isinstance(M.tensor_module(1, 1), ReflexiveModule_tensor)
        True
    """

    def tensor_factors(self):
        r"""
        Return the tensor factors of this tensor module.

        EXAMPLES::

            sage: M = FiniteRankFreeModule(ZZ, 3, name='M')
            sage: T = M.tensor_module(2, 3)
            sage: T.tensor_factors()
            [Rank-3 free module M over the Integer Ring,
             Rank-3 free module M over the Integer Ring,
             Dual of the Rank-3 free module M over the Integer Ring,
             Dual of the Rank-3 free module M over the Integer Ring,
             Dual of the Rank-3 free module M over the Integer Ring]
        """
        tensor_type = self.tensor_type()
        if tensor_type == (0,1):  # case of the dual
            raise NotImplementedError
        bmodule = self.base_module()
        factors = [bmodule] * tensor_type[0]
        dmodule = bmodule.dual()
        if tensor_type[1]:
            factors += [dmodule] * tensor_type[1]
        return factors



# sage_setup: distribution = sagemath-flint
# distutils: libraries = flint
# distutils: depends = flint/fq_zech.h

################################################################################
# This file is auto-generated by the script
#   SAGE_ROOT/src/sage_setup/autogen/flint_autogen.py.
# From the commit 3e2c3a3e091106a25ca9c6fba28e02f2cbcd654a
# Do not modify by hand! Fix and rerun the script instead.
################################################################################

from libc.stdio cimport FILE
from sage.libs.gmp.types cimport *
from sage.libs.mpfr.types cimport *
from sage.libs.flint.types cimport *

cdef extern from "flint_wrap.h":
    void fq_zech_ctx_init(fq_zech_ctx_t ctx, const fmpz_t p, slong d, const char * var) noexcept
    int _fq_zech_ctx_init_conway(fq_zech_ctx_t ctx, const fmpz_t p, slong d, const char * var) noexcept
    void fq_zech_ctx_init_conway(fq_zech_ctx_t ctx, const fmpz_t p, slong d, const char * var) noexcept
    void fq_zech_ctx_init_random(fq_zech_ctx_t ctx, const fmpz_t p, slong d, const char * var) noexcept
    void fq_zech_ctx_init_modulus(fq_zech_ctx_t ctx, const nmod_poly_t modulus, const char * var) noexcept
    int fq_zech_ctx_init_modulus_check(fq_zech_ctx_t ctx, const nmod_poly_t modulus, const char * var) noexcept
    void fq_zech_ctx_init_fq_nmod_ctx(fq_zech_ctx_t ctx, fq_nmod_ctx_t ctxn) noexcept
    int fq_zech_ctx_init_fq_nmod_ctx_check(fq_zech_ctx_t ctx, fq_nmod_ctx_t ctxn) noexcept
    void fq_zech_ctx_clear(fq_zech_ctx_t ctx) noexcept
    const nmod_poly_struct* fq_zech_ctx_modulus(const fq_zech_ctx_t ctx) noexcept
    slong fq_zech_ctx_degree(const fq_zech_ctx_t ctx) noexcept
    fmpz * fq_zech_ctx_prime(const fq_zech_ctx_t ctx) noexcept
    void fq_zech_ctx_order(fmpz_t f, const fq_zech_ctx_t ctx) noexcept
    mp_limb_t fq_zech_ctx_order_ui(const fq_zech_ctx_t ctx) noexcept
    int fq_zech_ctx_fprint(FILE * file, const fq_zech_ctx_t ctx) noexcept
    void fq_zech_ctx_print(const fq_zech_ctx_t ctx) noexcept
    void fq_zech_ctx_randtest(fq_zech_ctx_t ctx, flint_rand_t state) noexcept
    void fq_zech_ctx_randtest_reducible(fq_zech_ctx_t ctx, flint_rand_t state) noexcept
    void fq_zech_init(fq_zech_t rop, const fq_zech_ctx_t ctx) noexcept
    void fq_zech_init2(fq_zech_t rop, const fq_zech_ctx_t ctx) noexcept
    void fq_zech_clear(fq_zech_t rop, const fq_zech_ctx_t ctx) noexcept
    void _fq_zech_sparse_reduce(mp_ptr R, slong lenR, const fq_zech_ctx_t ctx) noexcept
    void _fq_zech_dense_reduce(mp_ptr R, slong lenR, const fq_zech_ctx_t ctx) noexcept
    void _fq_zech_reduce(mp_ptr r, slong lenR, const fq_zech_ctx_t ctx) noexcept
    void fq_zech_reduce(fq_zech_t rop, const fq_zech_ctx_t ctx) noexcept
    void fq_zech_add(fq_zech_t rop, const fq_zech_t op1, const fq_zech_t op2, const fq_zech_ctx_t ctx) noexcept
    void fq_zech_sub(fq_zech_t rop, const fq_zech_t op1, const fq_zech_t op2, const fq_zech_ctx_t ctx) noexcept
    void fq_zech_sub_one(fq_zech_t rop, const fq_zech_t op1, const fq_zech_ctx_t ctx) noexcept
    void fq_zech_neg(fq_zech_t rop, const fq_zech_t op, const fq_zech_ctx_t ctx) noexcept
    void fq_zech_mul(fq_zech_t rop, const fq_zech_t op1, const fq_zech_t op2, const fq_zech_ctx_t ctx) noexcept
    void fq_zech_mul_fmpz(fq_zech_t rop, const fq_zech_t op, const fmpz_t x, const fq_zech_ctx_t ctx) noexcept
    void fq_zech_mul_si(fq_zech_t rop, const fq_zech_t op, slong x, const fq_zech_ctx_t ctx) noexcept
    void fq_zech_mul_ui(fq_zech_t rop, const fq_zech_t op, ulong x, const fq_zech_ctx_t ctx) noexcept
    void fq_zech_sqr(fq_zech_t rop, const fq_zech_t op, const fq_zech_ctx_t ctx) noexcept
    void fq_zech_div(fq_zech_t rop, const fq_zech_t op1, const fq_zech_t op2, const fq_zech_ctx_t ctx) noexcept
    void _fq_zech_inv(mp_ptr * rop, mp_srcptr * op, slong len, const fq_zech_ctx_t ctx) noexcept
    void fq_zech_inv(fq_zech_t rop, const fq_zech_t op, const fq_zech_ctx_t ctx) noexcept
    void fq_zech_gcdinv(fq_zech_t f, fq_zech_t inv, const fq_zech_t op, const fq_zech_ctx_t ctx) noexcept
    void _fq_zech_pow(fmpz * rop, const fmpz * op, slong len, const fmpz_t e, const fmpz * a, const slong * j, slong lena, const fmpz_t p) noexcept
    void fq_zech_pow(fq_zech_t rop, const fq_zech_t op, const fmpz_t e, const fq_zech_ctx_t ctx) noexcept
    void fq_zech_pow_ui(fq_zech_t rop, const fq_zech_t op, const ulong e, const fq_zech_ctx_t ctx) noexcept
    int fq_zech_sqrt(fq_zech_t rop, const fq_zech_t op1, const fq_zech_ctx_t ctx) noexcept
    void fq_zech_pth_root(fq_zech_t rop, const fq_zech_t op1, const fq_zech_ctx_t ctx) noexcept
    bint fq_zech_is_square(const fq_zech_t op, const fq_zech_ctx_t ctx) noexcept
    int fq_zech_fprint_pretty(FILE * file, const fq_zech_t op, const fq_zech_ctx_t ctx) noexcept
    void fq_zech_print_pretty(const fq_zech_t op, const fq_zech_ctx_t ctx) noexcept
    int fq_zech_fprint(FILE * file, const fq_zech_t op, const fq_zech_ctx_t ctx) noexcept
    void fq_zech_print(const fq_zech_t op, const fq_zech_ctx_t ctx) noexcept
    char * fq_zech_get_str(const fq_zech_t op, const fq_zech_ctx_t ctx) noexcept
    char * fq_zech_get_str_pretty(const fq_zech_t op, const fq_zech_ctx_t ctx) noexcept
    void fq_zech_randtest(fq_zech_t rop, flint_rand_t state, const fq_zech_ctx_t ctx) noexcept
    void fq_zech_randtest_not_zero(fq_zech_t rop, flint_rand_t state, const fq_zech_ctx_t ctx) noexcept
    void fq_zech_randtest_dense(fq_zech_t rop, flint_rand_t state, const fq_zech_ctx_t ctx) noexcept
    void fq_zech_rand(fq_zech_t rop, flint_rand_t state, const fq_zech_ctx_t ctx) noexcept
    void fq_zech_rand_not_zero(fq_zech_t rop, flint_rand_t state, const fq_zech_ctx_t ctx) noexcept
    void fq_zech_set(fq_zech_t rop, const fq_zech_t op, const fq_zech_ctx_t ctx) noexcept
    void fq_zech_set_si(fq_zech_t rop, const slong x, const fq_zech_ctx_t ctx) noexcept
    void fq_zech_set_ui(fq_zech_t rop, const ulong x, const fq_zech_ctx_t ctx) noexcept
    void fq_zech_set_fmpz(fq_zech_t rop, const fmpz_t x, const fq_zech_ctx_t ctx) noexcept
    void fq_zech_swap(fq_zech_t op1, fq_zech_t op2, const fq_zech_ctx_t ctx) noexcept
    void fq_zech_zero(fq_zech_t rop, const fq_zech_ctx_t ctx) noexcept
    void fq_zech_one(fq_zech_t rop, const fq_zech_ctx_t ctx) noexcept
    void fq_zech_gen(fq_zech_t rop, const fq_zech_ctx_t ctx) noexcept
    int fq_zech_get_fmpz(fmpz_t rop, const fq_zech_t op, const fq_zech_ctx_t ctx) noexcept
    void fq_zech_get_fq_nmod(fq_nmod_t rop, const fq_zech_t op, const fq_zech_ctx_t ctx) noexcept
    void fq_zech_set_fq_nmod(fq_zech_t rop, const fq_nmod_t op, const fq_zech_ctx_t ctx) noexcept
    void fq_zech_get_nmod_poly(nmod_poly_t a, const fq_zech_t b, const fq_zech_ctx_t ctx) noexcept
    void fq_zech_set_nmod_poly(fq_zech_t a, const nmod_poly_t b, const fq_zech_ctx_t ctx) noexcept
    void fq_zech_get_nmod_mat(nmod_mat_t col, const fq_zech_t a, const fq_zech_ctx_t ctx) noexcept
    void fq_zech_set_nmod_mat(fq_zech_t a, const nmod_mat_t col, const fq_zech_ctx_t ctx) noexcept
    bint fq_zech_is_zero(const fq_zech_t op, const fq_zech_ctx_t ctx) noexcept
    bint fq_zech_is_one(const fq_zech_t op, const fq_zech_ctx_t ctx) noexcept
    bint fq_zech_equal(const fq_zech_t op1, const fq_zech_t op2, const fq_zech_ctx_t ctx) noexcept
    bint fq_zech_is_invertible(const fq_zech_t op, const fq_zech_ctx_t ctx) noexcept
    bint fq_zech_is_invertible_f(fq_zech_t f, const fq_zech_t op, const fq_zech_ctx_t ctx) noexcept
    void fq_zech_trace(fmpz_t rop, const fq_zech_t op, const fq_zech_ctx_t ctx) noexcept
    void fq_zech_norm(fmpz_t rop, const fq_zech_t op, const fq_zech_ctx_t ctx) noexcept
    void fq_zech_frobenius(fq_zech_t rop, const fq_zech_t op, slong e, const fq_zech_ctx_t ctx) noexcept
    int fq_zech_multiplicative_order(fmpz * ord, const fq_zech_t op, const fq_zech_ctx_t ctx) noexcept
    bint fq_zech_is_primitive(const fq_zech_t op, const fq_zech_ctx_t ctx) noexcept
    void fq_zech_bit_pack(fmpz_t f, const fq_zech_t op, flint_bitcnt_t bit_size, const fq_zech_ctx_t ctx) noexcept
    void fq_zech_bit_unpack(fq_zech_t rop, const fmpz_t f, flint_bitcnt_t bit_size, const fq_zech_ctx_t ctx) noexcept



# sage_setup: distribution = sagemath-flint
# distutils: libraries = flint
# distutils: depends = flint/gr_mpoly.h

################################################################################
# This file is auto-generated by the script
#   SAGE_ROOT/src/sage_setup/autogen/flint_autogen.py.
# From the commit 3e2c3a3e091106a25ca9c6fba28e02f2cbcd654a
# Do not modify by hand! Fix and rerun the script instead.
################################################################################

from libc.stdio cimport FILE
from sage.libs.gmp.types cimport *
from sage.libs.mpfr.types cimport *
from sage.libs.flint.types cimport *

cdef extern from "flint_wrap.h":
    void gr_mpoly_init(gr_mpoly_t A, const mpoly_ctx_t mctx, gr_ctx_t cctx) noexcept
    void gr_mpoly_init3(gr_mpoly_t A, slong alloc, flint_bitcnt_t bits, const mpoly_ctx_t mctx, gr_ctx_t cctx) noexcept
    void gr_mpoly_init2(gr_mpoly_t A, slong alloc, const mpoly_ctx_t mctx, gr_ctx_t cctx) noexcept
    void gr_mpoly_clear(gr_mpoly_t A, const mpoly_ctx_t mctx, gr_ctx_t cctx) noexcept
    void gr_mpoly_swap(gr_mpoly_t A, gr_mpoly_t B, const mpoly_ctx_t mctx, gr_ctx_t cctx) noexcept
    int gr_mpoly_set(gr_mpoly_t A, const gr_mpoly_t B, const mpoly_ctx_t mctx, gr_ctx_t cctx) noexcept
    int gr_mpoly_zero(gr_mpoly_t A, const mpoly_ctx_t mctx, gr_ctx_t cctx) noexcept
    truth_t gr_mpoly_is_zero(const gr_mpoly_t A, const mpoly_ctx_t mctx, gr_ctx_t cctx) noexcept
    int gr_mpoly_gen(gr_mpoly_t A, slong var, const mpoly_ctx_t mctx, gr_ctx_t cctx) noexcept
    truth_t gr_mpoly_is_gen(const gr_mpoly_t A, slong var, const mpoly_ctx_t mctx, gr_ctx_t cctx) noexcept
    truth_t gr_mpoly_equal(const gr_mpoly_t A, const gr_mpoly_t B, const mpoly_ctx_t mctx, gr_ctx_t cctx) noexcept
    int gr_mpoly_randtest_bits(gr_mpoly_t A, flint_rand_t state, slong length, flint_bitcnt_t exp_bits, const mpoly_ctx_t mctx, gr_ctx_t cctx) noexcept
    int gr_mpoly_write_pretty(gr_stream_t out, const gr_mpoly_t A, const char ** x, const mpoly_ctx_t mctx, gr_ctx_t cctx) noexcept
    int gr_mpoly_print_pretty(const gr_mpoly_t A, const char ** x, const mpoly_ctx_t mctx, gr_ctx_t cctx) noexcept
    int gr_mpoly_get_coeff_scalar_fmpz(gr_ptr c, const gr_mpoly_t A, const fmpz * exp, const mpoly_ctx_t mctx, gr_ctx_t cctx) noexcept
    int gr_mpoly_get_coeff_scalar_ui(gr_ptr c, const gr_mpoly_t A, const ulong * exp, const mpoly_ctx_t mctx, gr_ctx_t cctx) noexcept
    int gr_mpoly_set_coeff_scalar_fmpz(gr_mpoly_t A, gr_srcptr c, const fmpz * exp, const mpoly_ctx_t mctx, gr_ctx_t cctx) noexcept
    int gr_mpoly_set_coeff_ui_fmpz(gr_mpoly_t A, ulong c, const fmpz * exp, const mpoly_ctx_t mctx, gr_ctx_t cctx) noexcept
    int gr_mpoly_set_coeff_si_fmpz(gr_mpoly_t A, slong c, const fmpz * exp, const mpoly_ctx_t mctx, gr_ctx_t cctx) noexcept
    int gr_mpoly_set_coeff_fmpz_fmpz(gr_mpoly_t A, const fmpz_t c, const fmpz * exp, const mpoly_ctx_t mctx, gr_ctx_t cctx) noexcept
    int gr_mpoly_set_coeff_fmpq_fmpz(gr_mpoly_t A, const fmpq_t c, const fmpz * exp, const mpoly_ctx_t mctx, gr_ctx_t cctx) noexcept
    int gr_mpoly_set_coeff_scalar_ui(gr_mpoly_t poly, gr_srcptr c, const ulong * exp, const mpoly_ctx_t mctx, gr_ctx_t cctx) noexcept
    int gr_mpoly_set_coeff_ui_ui(gr_mpoly_t A, ulong c, const ulong * exp, const mpoly_ctx_t mctx, gr_ctx_t cctx) noexcept
    int gr_mpoly_set_coeff_si_ui(gr_mpoly_t A, slong c, const ulong * exp, const mpoly_ctx_t mctx, gr_ctx_t cctx) noexcept
    int gr_mpoly_set_coeff_fmpz_ui(gr_mpoly_t A, const fmpz_t c, const ulong * exp, const mpoly_ctx_t mctx, gr_ctx_t cctx) noexcept
    int gr_mpoly_set_coeff_fmpq_ui(gr_mpoly_t A, const fmpq_t c, const ulong * exp, const mpoly_ctx_t mctx, gr_ctx_t cctx) noexcept
    int gr_mpoly_neg(gr_mpoly_t A, const gr_mpoly_t B, const mpoly_ctx_t mctx, gr_ctx_t cctx) noexcept
    int gr_mpoly_add(gr_mpoly_t A, const gr_mpoly_t B, const gr_mpoly_t C, const mpoly_ctx_t mctx, gr_ctx_t cctx) noexcept
    int gr_mpoly_sub(gr_mpoly_t A, const gr_mpoly_t B, const gr_mpoly_t C, const mpoly_ctx_t mctx, gr_ctx_t cctx) noexcept
    int gr_mpoly_mul(gr_mpoly_t A, const gr_mpoly_t B, const gr_mpoly_t C, const mpoly_ctx_t mctx, gr_ctx_t cctx) noexcept
    int gr_mpoly_mul_johnson(gr_mpoly_t A, const gr_mpoly_t B, const gr_mpoly_t C, const mpoly_ctx_t mctx, gr_ctx_t cctx) noexcept
    int gr_mpoly_mul_monomial(gr_mpoly_t A, const gr_mpoly_t B, const gr_mpoly_t C, const mpoly_ctx_t mctx, gr_ctx_t cctx) noexcept
    int gr_mpoly_mul_scalar(gr_mpoly_t A, const gr_mpoly_t B, gr_srcptr c, const mpoly_ctx_t mctx, gr_ctx_t cctx) noexcept
    int gr_mpoly_mul_si(gr_mpoly_t A, const gr_mpoly_t B, slong c, const mpoly_ctx_t mctx, gr_ctx_t cctx) noexcept
    int gr_mpoly_mul_ui(gr_mpoly_t A, const gr_mpoly_t B, ulong c, const mpoly_ctx_t mctx, gr_ctx_t cctx) noexcept
    int gr_mpoly_mul_fmpz(gr_mpoly_t A, const gr_mpoly_t B, const fmpz_t c, const mpoly_ctx_t mctx, gr_ctx_t cctx) noexcept
    int gr_mpoly_mul_fmpq(gr_mpoly_t A, const gr_mpoly_t B, const fmpq_t c, const mpoly_ctx_t mctx, gr_ctx_t cctx) noexcept
    void _gr_mpoly_fit_length(gr_ptr * coeffs, slong * coeffs_alloc, ulong ** exps, slong * exps_alloc, slong N, slong length, gr_ctx_t cctx) noexcept
    void gr_mpoly_fit_length(gr_mpoly_t A, slong len, const mpoly_ctx_t mctx, gr_ctx_t cctx) noexcept
    void gr_mpoly_fit_bits(gr_mpoly_t A, flint_bitcnt_t bits, const mpoly_ctx_t mctx, gr_ctx_t cctx) noexcept
    void gr_mpoly_fit_length_fit_bits(gr_mpoly_t A, slong len, flint_bitcnt_t bits, const mpoly_ctx_t mctx, gr_ctx_t cctx) noexcept
    void gr_mpoly_fit_length_reset_bits(gr_mpoly_t A, slong len, flint_bitcnt_t bits, const mpoly_ctx_t mctx, gr_ctx_t cctx) noexcept
    void _gr_mpoly_set_length(gr_mpoly_t A, slong newlen, const mpoly_ctx_t mctx, gr_ctx_t cctx) noexcept
    void _gr_mpoly_push_exp_ui(gr_mpoly_t A, const ulong * exp, const mpoly_ctx_t mctx, gr_ctx_t cctx) noexcept
    int gr_mpoly_push_term_scalar_ui(gr_mpoly_t A, gr_srcptr c, const ulong * exp, const mpoly_ctx_t mctx, gr_ctx_t cctx) noexcept
    void _gr_mpoly_push_exp_fmpz(gr_mpoly_t A, const fmpz * exp, const mpoly_ctx_t mctx, gr_ctx_t cctx) noexcept
    int gr_mpoly_push_term_scalar_fmpz(gr_mpoly_t A, gr_srcptr c, const fmpz * exp, const mpoly_ctx_t mctx, gr_ctx_t cctx) noexcept
    void gr_mpoly_sort_terms(gr_mpoly_t A, const mpoly_ctx_t mctx, gr_ctx_t cctx) noexcept
    int gr_mpoly_combine_like_terms(gr_mpoly_t A, const mpoly_ctx_t mctx, gr_ctx_t cctx) noexcept
    truth_t gr_mpoly_is_canonical(const gr_mpoly_t A, const mpoly_ctx_t mctx, gr_ctx_t cctx) noexcept
    void gr_mpoly_assert_canonical(const gr_mpoly_t A, const mpoly_ctx_t mctx, gr_ctx_t cctx) noexcept



# sage_setup: distribution = sagemath-gap
"""
Constructor for permutations

This module contains the generic constructor to build element of the symmetric
groups (or more general permutation groups) called
:class:`~sage.groups.perm_gps.permgroup_element.PermutationGroupElement`. These
objects have a more group theoretic flavor than the more combinatorial
:class:`~sage.combinat.permutation.Permutation`.
"""
# ****************************************************************************
#       Copyright (C) 2006 William Stein <wstein@gmail.com>
#       Copyright (C) 2006 David Joyner
#       Copyright (C) 2019 Vincent Delecroix <20100.delecroix@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#                  https://www.gnu.org/licenses/
# ****************************************************************************

from sage.groups.perm_gps import permgroup_element
from sage.interfaces.gap import GapElement
from sage.libs.gap.element import GapElement_Permutation
from sage.misc.lazy_import import lazy_import

lazy_import('sage.combinat.permutation', ['Permutation', 'from_cycles'])
lazy_import('sage.misc.sage_eval', 'sage_eval')

try:
    from cypari2.gen import Gen as pari_gen
except ImportError:
    pari_gen = ()


def PermutationGroupElement(g, parent=None, check=True):
    r"""
    Build a permutation from ``g``.

    INPUT:

    - ``g`` -- either

      - a list of images

      - a tuple describing a single cycle

      - a list of tuples describing the cycle decomposition

      - a string describing the cycle decomposition

    - ``parent`` -- (optional) an ambient permutation group for the result;
      it is mandatory if you want a permutation on a domain different
      from `\{1, \ldots, n\}`

    - ``check`` -- boolean (default: ``True``); whether additional check are
      performed. Setting it to ``False`` is likely to result in faster code.

    EXAMPLES:

    Initialization as a list of images::

        sage: p = PermutationGroupElement([1,4,2,3])
        sage: p
        (2,4,3)
        sage: p.parent()
        Symmetric group of order 4! as a permutation group

    Initialization as a list of cycles::

        sage: p = PermutationGroupElement([(3,5),(4,6,9)])
        sage: p
        (3,5)(4,6,9)
        sage: p.parent()
        Symmetric group of order 9! as a permutation group

    Initialization as a string representing a cycle decomposition::

        sage: p = PermutationGroupElement('(2,4)(3,5)')
        sage: p
        (2,4)(3,5)
        sage: p.parent()
        Symmetric group of order 5! as a permutation group

    By default the constructor assumes that the domain is `\{1, \dots, n\}`
    but it can be set to anything via its second ``parent`` argument::

        sage: S = SymmetricGroup(['a', 'b', 'c', 'd', 'e'])
        sage: PermutationGroupElement(['e', 'c', 'b', 'a', 'd'], S)
        ('a','e','d')('b','c')
        sage: PermutationGroupElement(('a', 'b', 'c'), S)
        ('a','b','c')
        sage: PermutationGroupElement([('a', 'c'), ('b', 'e')], S)
        ('a','c')('b','e')
        sage: PermutationGroupElement("('a','b','e')('c','d')", S)
        ('a','b','e')('c','d')

    But in this situation, you might want to use the more direct::

        sage: S(['e', 'c', 'b', 'a', 'd'])
        ('a','e','d')('b','c')
        sage: S(('a', 'b', 'c'))
        ('a','b','c')
        sage: S([('a', 'c'), ('b', 'e')])
        ('a','c')('b','e')
        sage: S("('a','b','e')('c','d')")
        ('a','b','e')('c','d')
    """
    if isinstance(g, permgroup_element.PermutationGroupElement):
        if parent is None or g.parent() is parent:
            return g

    if parent is None:
        from sage.groups.perm_gps.permgroup_named import SymmetricGroup

        try:
            v = standardize_generator(g, None)
        except KeyError:
            raise ValueError("invalid permutation vector: %s" % g)
        parent = SymmetricGroup(len(v))
        # We have constructed the parent from the element and already checked
        #   that it is a valid permutation
        check = False

    return parent.element_class(g, parent, check)


def string_to_tuples(g):
    """
    EXAMPLES::

        sage: from sage.groups.perm_gps.constructor import string_to_tuples
        sage: string_to_tuples('(1,2,3)')
        [(1, 2, 3)]
        sage: string_to_tuples('(1,2,3)(4,5)')
        [(1, 2, 3), (4, 5)]
        sage: string_to_tuples(' (1,2, 3) (4,5)')
        [(1, 2, 3), (4, 5)]
        sage: string_to_tuples('(1,2)(3)')
        [(1, 2), (3,)]
    """
    if not isinstance(g, str):
        raise ValueError("g (= %s) must be a string" % g)
    elif g == '()':
        return []
    g = g.replace('\n', '').replace(' ', '').replace(')(', '),(').replace(')', ',)')
    g = '[' + g + ']'
    return sage_eval(g, preparse=False)


def standardize_generator(g, convert_dict=None, as_cycles=False):
    r"""
    Standardize the input for permutation group elements to a list
    or a list of tuples.

    This was factored out of the
    ``PermutationGroupElement.__init__`` since
    ``PermutationGroup_generic.__init__`` needs to do the same computation
    in order to compute the domain of a group when it's not explicitly
    specified.

    INPUT:

    - ``g`` -- a :class:`list`, :class:`tuple`, :class:`string`, :class:`GapElement`,
      :class:`PermutationGroupElement`, or :class:`Permutation`

    - ``convert_dict`` -- (optional) a dictionary used to convert the
      points to a number compatible with GAP

    - ``as_cycles`` -- boolean (default: ``False``); whether the output should be
      as cycles or in one-line notation

    OUTPUT: the permutation in as a list in one-line notation or a list of
    cycles as tuples

    EXAMPLES::

        sage: from sage.groups.perm_gps.constructor import standardize_generator
        sage: standardize_generator('(1,2)')
        [2, 1]

        sage: p = PermutationGroupElement([(1,2)])
        sage: standardize_generator(p)
        [2, 1]
        sage: standardize_generator(p._gap_())
        [2, 1]
        sage: standardize_generator((1,2))
        [2, 1]
        sage: standardize_generator([(1,2)])
        [2, 1]

        sage: standardize_generator(p, as_cycles=True)
        [(1, 2)]
        sage: standardize_generator(p._gap_(), as_cycles=True)
        [(1, 2)]
        sage: standardize_generator((1,2), as_cycles=True)
        [(1, 2)]
        sage: standardize_generator([(1,2)], as_cycles=True)
        [(1, 2)]

        sage: standardize_generator(Permutation([2,1,3]))
        [2, 1, 3]
        sage: standardize_generator(Permutation([2,1,3]), as_cycles=True)
        [(1, 2), (3,)]

    ::

        sage: d = {'a': 1, 'b': 2}
        sage: p = SymmetricGroup(['a', 'b']).gen(0); p
        ('a','b')
        sage: standardize_generator(p, convert_dict=d)
        [2, 1]
        sage: standardize_generator(p._gap_(), convert_dict=d)
        [2, 1]
        sage: standardize_generator(('a','b'), convert_dict=d)
        [2, 1]
        sage: standardize_generator([('a','b')], convert_dict=d)
        [2, 1]

        sage: standardize_generator(p, convert_dict=d, as_cycles=True)
        [(1, 2)]
        sage: standardize_generator(p._gap_(), convert_dict=d, as_cycles=True)
        [(1, 2)]
        sage: standardize_generator(('a','b'), convert_dict=d, as_cycles=True)
        [(1, 2)]
        sage: standardize_generator([('a','b')], convert_dict=d, as_cycles=True)
        [(1, 2)]
    """
    if isinstance(g, pari_gen):
        g = list(g)

    needs_conversion = True

    if isinstance(g, GapElement_Permutation):
        g = g.sage()
        needs_conversion = False
    if isinstance(g, GapElement):
        g = str(g)
        needs_conversion = False
    if isinstance(g, Permutation):
        if as_cycles:
            return g.cycle_tuples()
        return g._list
    elif isinstance(g, permgroup_element.PermutationGroupElement):
        if not as_cycles:
            l = list(range(1, g.parent().degree() + 1))
            return g._act_on_list_on_position(l)
        g = g.cycle_tuples()
    elif isinstance(g, str):
        g = string_to_tuples(g)
    elif isinstance(g, tuple) and (len(g) == 0 or not isinstance(g[0], tuple)):
        g = [g]

    # Get the permutation in list notation
    if isinstance(g, list) and not (g and isinstance(g[0], tuple)):
        if convert_dict is not None and needs_conversion:
            for i, x in enumerate(g):
                g[i] = convert_dict[x]
        if as_cycles:
            return Permutation(g).cycle_tuples()
        return g

    # Otherwise it is in cycle notation
    if convert_dict is not None and needs_conversion:
        g = [tuple([convert_dict[x] for x in cycle]) for cycle in g]
    if not as_cycles:
        degree = max([1] + [max(cycle + (1,)) for cycle in g])
        g = from_cycles(degree, g)
    return g



# from sage.misc.all__sagemath_objects import *
from sage.misc.all__sagemath_environment import *
from sage.misc.all__sagemath_modules import *
from sage.misc.all__sagemath_repl import *

lazy_import('sage.misc.banner', 'banner', deprecation=34259)
lazy_import('sage.misc.edit_module', 'set_edit_template', deprecation=34259)
lazy_import('sage.misc.profiler', 'Profiler', deprecation=34259)
lazy_import('sage.misc.trace', 'trace', deprecation=34259)
lazy_import('sage.misc.package', ('installed_packages', 'is_package_installed',
                                  'package_versions'),
            deprecation=34259)
lazy_import('sage.misc.benchmark', 'benchmark', deprecation=34259)
lazy_import('sage.repl.interpreter', 'logstr', deprecation=34259)

# Following will go to all__sagemath_objects.py in #36566
from sage.misc.randstate import seed, set_random_seed, initial_seed, current_randstate
from sage.misc.prandom import *
from sage.misc.sage_timeit_class import timeit
from sage.misc.session import load_session, save_session, show_identifiers
from sage.misc.reset import reset, restore



# sage_setup: distribution = sagemath-modules
from sage.libs.gmp.types cimport mpz_t
from sage.modules.free_module_element cimport FreeModuleElement
from sage.structure.parent cimport Parent


cdef class Vector_integer_dense(FreeModuleElement):
    cdef mpz_t* _entries
    cdef int _init(self, Py_ssize_t degree, Parent parent) except -1

    cdef inline Vector_integer_dense _new_c(self):
        cdef type t = type(self)
        cdef Vector_integer_dense x = <Vector_integer_dense>(t.__new__(t))
        x._init(self._degree, self._parent)
        return x



# sage_setup: distribution = sagemath-flint
from typing import Any

from sage.libs.mpfi.types import mpfi_t
from sage.libs.mpfr.types import mpfr_prec_t
from sage.rings.real_mpfi import RealIntervalFieldElement, RealIntervalField_class
from sage.structure.element import FieldElement

class ComplexIntervalFieldElement(FieldElement):
    __re: mpfi_t
    __im: mpfi_t
    _prec: mpfr_prec_t
    _multiplicative_order: Any

    def _new(self) -> 'ComplexIntervalFieldElement':
        ...

    def _new_real(self) -> RealIntervalFieldElement:
        ...
